@@ -2,15 +2,14 @@
  *
  * HISTOGRAM.C -  Nagios Alert Histogram CGI
  *
- * Copyright (c) 2001-2002 Ethan Galstad (nagios@nagios.org)
- * Last Modified: 11-22-2002
+ * Copyright (c) 2001-2006 Ethan Galstad (nagios@nagios.org)
+ * Last Modified: 01-20-2006
  *
  * License:
  * 
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU General Public License version 2 as
+ * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -22,15 +21,14 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *************************************************************************/
 
-#include "../common/config.h"
-#include "../common/locations.h"
-#include "../common/common.h"
-#include "../common/objects.h"
-#include "../common/statusdata.h"
+#include "../include/config.h"
+#include "../include/common.h"
+#include "../include/objects.h"
+#include "../include/statusdata.h"
 
-#include "cgiutils.h"
-#include "getcgi.h"
-#include "auth.h"
+#include "../include/cgiutils.h"
+#include "../include/getcgi.h"
+#include "../include/cgiauth.h"
 
 #include <gd.h>			/* Boutell's GD library function */
 #include <gdfonts.h>		/* GD library small font definition */
@@ -90,6 +88,7 @@
 #define TIMEPERIOD_LASTYEAR	10
 #define TIMEPERIOD_LAST24HOURS	11
 #define TIMEPERIOD_LAST7DAYS	12
+#define TIMEPERIOD_LAST31DAYS   13
 
 
 #define MAX_ARCHIVE_SPREAD	65
@@ -130,11 +129,11 @@ extern char url_images_path[MAX_FILENAME_LENGTH];
 extern char url_stylesheets_path[MAX_FILENAME_LENGTH];
 extern char physical_images_path[MAX_FILENAME_LENGTH];
 
+extern int     log_rotation_method;
+
 extern host *host_list;
 extern service *service_list;
 
-extern int     log_rotation_method;
-
 
 
 authdata current_authdata;
@@ -188,7 +187,7 @@ int display_type=DISPLAY_NO_HISTOGRAM;
 int mode=CREATE_HTML;
 int input_type=GET_INPUT_NONE;
 int timeperiod_type=TIMEPERIOD_LAST24HOURS;
-int breakdown_type=BREAKDOWN_DAY_OF_MONTH;
+int breakdown_type=BREAKDOWN_HOURLY;
 int compute_time_from_parts=FALSE;
 
 int initial_states_logged=FALSE;
@@ -258,11 +257,11 @@ int main(int argc, char **argv){
 	reset_cgi_vars();
 	
 	/* read the CGI configuration file */
-	result=read_cgi_config_file(DEFAULT_CGI_CONFIG_FILE);
+	result=read_cgi_config_file(get_cgi_config_location());
 	if(result==ERROR){
 		if(mode==CREATE_HTML){
 			document_header(FALSE);
-			cgi_config_file_error(DEFAULT_CGI_CONFIG_FILE);
+			cgi_config_file_error(get_cgi_config_location());
 			document_footer();
 		        }
 		return ERROR;
@@ -291,7 +290,7 @@ int main(int argc, char **argv){
                 }
 
 	/* read all status data */
-	result=read_all_status_data(DEFAULT_CGI_CONFIG_FILE,READ_ALL_STATUS_DATA);
+	result=read_all_status_data(get_cgi_config_location(),READ_ALL_STATUS_DATA);
 	if(result==ERROR){
 		if(mode==CREATE_HTML){
 			document_header(FALSE);
@@ -342,21 +341,25 @@ int main(int argc, char **argv){
 			printf("<TR><TD CLASS='linkBox'>\n");
 
 			if(display_type==DISPLAY_HOST_HISTOGRAM){
+#ifdef USE_TRENDS
 				printf("<a href='%s?host=%s&t1=%lu&t2=%lu&assumestateretention=%s'>View Trends For This Host</a><BR>\n",TRENDS_CGI,url_encode(host_name),t1,t2,(assume_state_retention==TRUE)?"yes":"no");
+#endif
 				printf("<a href='%s?host=%s&t1=%lu&t2=%lu&assumestateretention=%s&show_log_entries'>View Availability Report For This Host</a><BR>\n",AVAIL_CGI,url_encode(host_name),t1,t2,(assume_state_retention==TRUE)?"yes":"no");
 				printf("<a href='%s?host=%s'>View Status Detail For This Host</a><BR>\n",STATUS_CGI,url_encode(host_name));
 				printf("<a href='%s?host=%s'>View History For This Host</a><BR>\n",HISTORY_CGI,url_encode(host_name));
 				printf("<a href='%s?host=%s'>View Notifications For This Host</a><BR>\n",NOTIFICATIONS_CGI,url_encode(host_name));
 		                }
 			else{
+#ifdef USE_TRENDS
 				printf("<a href='%s?host=%s",TRENDS_CGI,url_encode(host_name));
+#endif
 				printf("&service=%s&t1=%lu&t2=%lu&assumestateretention=%s'>View Trends For This Service</a><BR>\n",url_encode(svc_description),t1,t2,(assume_state_retention==TRUE)?"yes":"no");
 				printf("<a href='%s?host=%s",AVAIL_CGI,url_encode(host_name));
 				printf("&service=%s&t1=%lu&t2=%lu&assumestateretention=%s&show_log_entries'>View Availability Report For This Service</a><BR>\n",url_encode(svc_description),t1,t2,(assume_state_retention==TRUE)?"yes":"no");
 				printf("<A HREF='%s?host=%s&",HISTORY_CGI,url_encode(host_name));
 				printf("service=%s'>View History This Service</A><BR>\n",url_encode(svc_description));
 				printf("<A HREF='%s?host=%s&",NOTIFICATIONS_CGI,url_encode(host_name));
-				printf("service=%s'>View Notifications This Service</A><BR>\n",url_encode(svc_description));
+				printf("service=%s'>View Notifications For This Service</A><BR>\n",url_encode(svc_description));
 		                }
 
 			printf("</TD></TR>\n");
@@ -379,7 +382,7 @@ int main(int argc, char **argv){
 
 			printf("<BR>\n");
 
-			printf("<IMG SRC='%s%s' BORDER=0 ALT='%s Event Histogram'>\n",url_images_path,TRENDS_ICON,(display_type==DISPLAY_HOST_HISTOGRAM)?"Host":"Service");
+			printf("<IMG SRC='%s%s' BORDER=0 ALT='%s Event Histogram' TITLE='%s Event Histogram'>\n",url_images_path,TRENDS_ICON,(display_type==DISPLAY_HOST_HISTOGRAM)?"Host":"Service",(display_type==DISPLAY_HOST_HISTOGRAM)?"Host":"Service");
 
 			printf("<BR CLEAR=ALL>\n");
 
@@ -411,17 +414,18 @@ int main(int argc, char **argv){
 			printf("<tr><td CLASS='optBoxItem' valign=top align=left>Report period:</td><td CLASS='optBoxItem' valign=top align=left>Assume state retention:</td></tr>\n");
 			printf("<tr><td CLASS='optBoxItem' valign=top align=left>\n");
 			printf("<select name='timeperiod'>\n");
-			printf("<option value=custom SELECTED>[ Current time range ]\n");
-			printf("<option value=last24hours>Last 24 Hours\n");
-			printf("<option value=today>Today\n");
-			printf("<option value=yesterday>Yesterday\n");
-			printf("<option value=thisweek>This Week\n");
-			printf("<option value=last7days>Last 7 Days\n");
-			printf("<option value=lastweek>Last Week\n");
-			printf("<option value=thismonth>This Month\n");
-			printf("<option value=lastmonth>Last Month\n");
-			printf("<option value=thisyear>This Year\n");
-			printf("<option value=lastyear>Last Year\n");
+			printf("<option value=custom>[ Current time range ]\n");
+			printf("<option value=today %s>Today\n",(timeperiod_type==TIMEPERIOD_TODAY)?"SELECTED":"");
+			printf("<option value=last24hours %s>Last 24 Hours\n",(timeperiod_type==TIMEPERIOD_LAST24HOURS)?"SELECTED":"");
+			printf("<option value=yesterday %s>Yesterday\n",(timeperiod_type==TIMEPERIOD_YESTERDAY)?"SELECTED":"");
+			printf("<option value=thisweek %s>This Week\n",(timeperiod_type==TIMEPERIOD_THISWEEK)?"SELECTED":"");
+			printf("<option value=last7days %s>Last 7 Days\n",(timeperiod_type==TIMEPERIOD_LAST7DAYS)?"SELECTED":"");
+			printf("<option value=lastweek %s>Last Week\n",(timeperiod_type==TIMEPERIOD_LASTWEEK)?"SELECTED":"");
+			printf("<option value=thismonth %s>This Month\n",(timeperiod_type==TIMEPERIOD_THISMONTH)?"SELECTED":"");
+			printf("<option value=last31days %s>Last 31 Days\n",(timeperiod_type==TIMEPERIOD_LAST31DAYS)?"SELECTED":"");
+			printf("<option value=lastmonth %s>Last Month\n",(timeperiod_type==TIMEPERIOD_LASTMONTH)?"SELECTED":"");
+			printf("<option value=thisyear %s>This Year\n",(timeperiod_type==TIMEPERIOD_THISYEAR)?"SELECTED":"");
+			printf("<option value=lastyear %s>Last Year\n",(timeperiod_type==TIMEPERIOD_LASTYEAR)?"SELECTED":"");
 			printf("</select>\n");
 			printf("</td><td CLASS='optBoxItem' valign=top align=left>\n");
 			printf("<select name='assumestateretention'>\n");
@@ -518,12 +522,12 @@ int main(int argc, char **argv){
 
 	/* check authorization... */
 	if(display_type==DISPLAY_HOST_HISTOGRAM){
-		temp_host=find_host(host_name,NULL);
+		temp_host=find_host(host_name);
 		if(temp_host==NULL || is_authorized_for_host(temp_host,&current_authdata)==FALSE)
 			is_authorized=FALSE;
 	        }
 	else if(display_type==DISPLAY_SERVICE_HISTOGRAM){
-		temp_service=find_service(host_name,svc_description,NULL);
+		temp_service=find_service(host_name,svc_description);
 		if(temp_service==NULL || is_authorized_for_service(temp_service,&current_authdata)==FALSE)
 			is_authorized=FALSE;
 	        }
@@ -686,7 +690,7 @@ int main(int argc, char **argv){
 			printf("<form method=\"GET\" action=\"%s\">\n",HISTOGRAM_CGI);
 			printf("<input type='hidden' name='input' value='getoptions'>\n");
 
-			printf("<tr><td class='reportSelectSubTitle' valign=center>Host</td>\n");
+			printf("<tr><td class='reportSelectSubTitle' valign=center>Host:</td>\n");
 			printf("<td class='reportSelectItem' valign=center>\n");
 			printf("<select name='host'>\n");
 
@@ -791,13 +795,14 @@ int main(int argc, char **argv){
 			printf("<tr><td class='reportSelectSubTitle' align=right>Report Period:</td>\n");
 			printf("<td class='reportSelectItem'>\n");
 			printf("<select name='timeperiod'>\n");
-			printf("<option value=last24hours>Last 24 Hours\n");
 			printf("<option value=today>Today\n");
+			printf("<option value=last24hours>Last 24 Hours\n");
 			printf("<option value=yesterday>Yesterday\n");
 			printf("<option value=thisweek>This Week\n");
 			printf("<option value=last7days SELECTED>Last 7 Days\n");
 			printf("<option value=lastweek>Last Week\n");
 			printf("<option value=thismonth>This Month\n");
+			printf("<option value=last31days>Last 31 Days\n");
 			printf("<option value=lastmonth>Last Month\n");
 			printf("<option value=thisyear>This Year\n");
 			printf("<option value=lastyear>Last Year\n");
@@ -805,7 +810,7 @@ int main(int argc, char **argv){
 			printf("</select>\n");
 			printf("</td></tr>\n");
 
-			printf("<tr><td colspan=2 valign=top calss='reportSelectSubTitle'><i>If Custom Report Period...</i></td></tr>\n");
+			printf("<tr><td valign=top calss='reportSelectSubTitle'>If Custom Report Period...</td></tr>\n");
 
 			printf("<tr>");
 			printf("<td valign=top class='reportSelectSubTitle'>Start Date (Inclusive):</td>\n");
@@ -942,8 +947,13 @@ int main(int argc, char **argv){
 			printf("<TABLE BORDER=0 cellpadding=5>\n");
 			printf("<form method=\"GET\" action=\"%s\">\n",HISTOGRAM_CGI);
 
-			printf("<tr><td class='reportSelectSubTitle'><input type='radio' name='input' value='gethost'></td><td class='reportSelectItem'>Host</td></tr>\n");
-			printf("<tr><td class='reportSelectSubTitle'><input type='radio' name='input' value='getservice'></td><td class='reportSelectItem'>Service</td></tr>\n");
+			printf("<tr><td class='reportSelectSubTitle' align=right>Type:</td>\n");
+			printf("<td class='reportSelectItem'>\n");
+			printf("<select name='input'>\n");
+			printf("<option value=gethost>Host\n");
+			printf("<option value=getservice>Service\n");
+			printf("</select>\n");
+			printf("</td></tr>\n");
 
 			printf("<tr><td></td><td class='reportSelectItem'>\n");
 			printf("<input type='submit' value='Continue to Step 2'>\n");
@@ -994,8 +1004,10 @@ void document_header(int use_stylesheet){
 		printf("Nagios Histogram\n");
 		printf("</title>\n");
 
-		if(use_stylesheet==TRUE)
+		if(use_stylesheet==TRUE){
+			printf("<LINK REL='stylesheet' TYPE='text/css' HREF='%s%s'>\n",url_stylesheets_path,COMMON_CSS);
 			printf("<LINK REL='stylesheet' TYPE='text/css' HREF='%s%s'>\n",url_stylesheets_path,HISTOGRAM_CSS);
+		        }
 	
 		printf("</head>\n");
 
@@ -1171,6 +1183,8 @@ int process_cgivars(void){
 				timeperiod_type=TIMEPERIOD_LAST24HOURS;
 			else if(!strcmp(variables[x],"last7days"))
 				timeperiod_type=TIMEPERIOD_LAST7DAYS;
+			else if(!strcmp(variables[x],"last31days"))
+				timeperiod_type=TIMEPERIOD_LAST31DAYS;
 			else if(!strcmp(variables[x],"custom"))
 				timeperiod_type=TIMEPERIOD_CUSTOM;
 			else
@@ -1454,7 +1468,7 @@ int process_cgivars(void){
 				assume_state_retention=FALSE;
 		        }
 
-		/* we found the initial states logges option */
+		/* we found the initial states logged option */
 		else if(!strcmp(variables[x],"initialstateslogged")){
 			x++;
 			if(variables[x]==NULL){
@@ -1638,7 +1652,7 @@ void graph_all_histogram_data(void){
 			snprintf(temp_buffer,sizeof(temp_buffer)-1,"%d",(int)(current_unit*y_units));
 			temp_buffer[sizeof(temp_buffer)-1]='\x0';
 			string_width=gdFontSmall->w*strlen(temp_buffer);
-			gdImageString(histogram_image,gdFontSmall,DRAWING_X_OFFSET-string_width-5,DRAWING_Y_OFFSET-(current_unit*y_units*y_scaling_factor)-(string_height/2),temp_buffer,color_black);
+			gdImageString(histogram_image,gdFontSmall,DRAWING_X_OFFSET-string_width-5,DRAWING_Y_OFFSET-(current_unit*y_units*y_scaling_factor)-(string_height/2),(unsigned char *)temp_buffer,color_black);
 	                }
 	        }
 
@@ -1660,14 +1674,14 @@ void graph_all_histogram_data(void){
 		temp_buffer[sizeof(temp_buffer)-1]='\x0';
 		string_width=gdFontSmall->w*strlen(temp_buffer);
 
-		gdImageStringUp(histogram_image,gdFontSmall,DRAWING_X_OFFSET+(current_unit*x_units)-(string_height/2),DRAWING_Y_OFFSET+5+string_width,temp_buffer,color_black);
+		gdImageStringUp(histogram_image,gdFontSmall,DRAWING_X_OFFSET+(current_unit*x_units)-(string_height/2),DRAWING_Y_OFFSET+5+string_width,(unsigned char *)temp_buffer,color_black);
 	        }
 
 	/* draw y unit measure */
 	snprintf(temp_buffer,sizeof(temp_buffer)-1,"Number of Events");
 	temp_buffer[sizeof(temp_buffer)-1]='\x0';
 	string_width=gdFontSmall->w*strlen(temp_buffer);
-	gdImageStringUp(histogram_image,gdFontSmall,0,DRAWING_Y_OFFSET-(DRAWING_HEIGHT/2)+(string_width/2),temp_buffer,color_black);
+	gdImageStringUp(histogram_image,gdFontSmall,0,DRAWING_Y_OFFSET-(DRAWING_HEIGHT/2)+(string_width/2),(unsigned char *)temp_buffer,color_black);
 
 	/* draw x unit measure */
 	if(breakdown_type==BREAKDOWN_MONTHLY)
@@ -1680,7 +1694,7 @@ void graph_all_histogram_data(void){
 		snprintf(temp_buffer,sizeof(temp_buffer)-1,"Hour of the Day (15 minute increments)");
 	temp_buffer[sizeof(temp_buffer)-1]='\x0';
 	string_width=gdFontSmall->w*strlen(temp_buffer);
-	gdImageString(histogram_image,gdFontSmall,DRAWING_X_OFFSET+(DRAWING_WIDTH/2)-(string_width/2),DRAWING_Y_OFFSET+70,temp_buffer,color_black);
+	gdImageString(histogram_image,gdFontSmall,DRAWING_X_OFFSET+(DRAWING_WIDTH/2)-(string_width/2),DRAWING_Y_OFFSET+70,(unsigned char *)temp_buffer,color_black);
 
 	/* draw title */
 	snprintf(start_time,sizeof(start_time)-1,"%s",ctime(&t1));
@@ -1954,60 +1968,60 @@ void graph_all_histogram_data(void){
 	snprintf(temp_buffer,sizeof(temp_buffer)-1,"EVENT TYPE");
 	temp_buffer[sizeof(temp_buffer)-1]='\x0';
 	string_width=gdFontSmall->w*strlen(temp_buffer);
-	gdImageString(histogram_image,gdFontSmall,DRAWING_X_OFFSET+DRAWING_WIDTH+15,DRAWING_Y_OFFSET-DRAWING_HEIGHT,temp_buffer,color_black);
+	gdImageString(histogram_image,gdFontSmall,DRAWING_X_OFFSET+DRAWING_WIDTH+15,DRAWING_Y_OFFSET-DRAWING_HEIGHT,(unsigned char *)temp_buffer,color_black);
 
 	snprintf(temp_buffer,sizeof(temp_buffer)-1,"  MIN   MAX   SUM   AVG");
 	temp_buffer[sizeof(temp_buffer)-1]='\x0';
 	string_width=gdFontSmall->w*strlen(temp_buffer);
-	gdImageString(histogram_image,gdFontSmall,DRAWING_X_OFFSET+DRAWING_WIDTH+115,DRAWING_Y_OFFSET-DRAWING_HEIGHT,temp_buffer,color_black);
+	gdImageString(histogram_image,gdFontSmall,DRAWING_X_OFFSET+DRAWING_WIDTH+115,DRAWING_Y_OFFSET-DRAWING_HEIGHT,(unsigned char *)temp_buffer,color_black);
 
 	draw_line(DRAWING_X_OFFSET+DRAWING_WIDTH+15,DRAWING_Y_OFFSET-DRAWING_HEIGHT+string_height+2,DRAWING_X_OFFSET+DRAWING_WIDTH+275,DRAWING_Y_OFFSET-DRAWING_HEIGHT+string_height+2,color_black);
 
 	snprintf(temp_buffer,sizeof(temp_buffer)-1,"Recovery (%s):",(display_type==DISPLAY_SERVICE_HISTOGRAM)?"Ok":"Up");
 	temp_buffer[sizeof(temp_buffer)-1]='\x0';
 	string_width=gdFontSmall->w*strlen(temp_buffer);
-	gdImageString(histogram_image,gdFontSmall,DRAWING_X_OFFSET+DRAWING_WIDTH+15,DRAWING_Y_OFFSET-DRAWING_HEIGHT+((string_height+5)*1),temp_buffer,color_green);
+	gdImageString(histogram_image,gdFontSmall,DRAWING_X_OFFSET+DRAWING_WIDTH+15,DRAWING_Y_OFFSET-DRAWING_HEIGHT+((string_height+5)*1),(unsigned char *)temp_buffer,color_green);
 
 	state1_avg=(double)((double)state1_sum/(double)total_buckets);
 	snprintf(temp_buffer,sizeof(temp_buffer)-1,"%5lu %5lu %5lu   %.2f",state1_min,state1_max,state1_sum,state1_avg);
 	temp_buffer[sizeof(temp_buffer)-1]='\x0';
 	string_width=gdFontSmall->w*strlen(temp_buffer);
-	gdImageString(histogram_image,gdFontSmall,DRAWING_X_OFFSET+DRAWING_WIDTH+115,DRAWING_Y_OFFSET-DRAWING_HEIGHT+((string_height+5)*1),temp_buffer,color_black);
+	gdImageString(histogram_image,gdFontSmall,DRAWING_X_OFFSET+DRAWING_WIDTH+115,DRAWING_Y_OFFSET-DRAWING_HEIGHT+((string_height+5)*1),(unsigned char *)temp_buffer,color_black);
 
 	snprintf(temp_buffer,sizeof(temp_buffer)-1,"%s:",(display_type==DISPLAY_SERVICE_HISTOGRAM)?"Warning":"Down");
 	temp_buffer[sizeof(temp_buffer)-1]='\x0';
 	string_width=gdFontSmall->w*strlen(temp_buffer);
-	gdImageString(histogram_image,gdFontSmall,DRAWING_X_OFFSET+DRAWING_WIDTH+15,DRAWING_Y_OFFSET-DRAWING_HEIGHT+((string_height+5)*2),temp_buffer,(display_type==DISPLAY_SERVICE_HISTOGRAM)?color_yellow:color_red);
+	gdImageString(histogram_image,gdFontSmall,DRAWING_X_OFFSET+DRAWING_WIDTH+15,DRAWING_Y_OFFSET-DRAWING_HEIGHT+((string_height+5)*2),(unsigned char *)temp_buffer,(display_type==DISPLAY_SERVICE_HISTOGRAM)?color_yellow:color_red);
 
 	state2_avg=(double)((double)state2_sum/(double)total_buckets);
 	snprintf(temp_buffer,sizeof(temp_buffer)-1,"%5lu %5lu %5lu   %.2f",state2_min,state2_max,state2_sum,state2_avg);
 	temp_buffer[sizeof(temp_buffer)-1]='\x0';
 	string_width=gdFontSmall->w*strlen(temp_buffer);
-	gdImageString(histogram_image,gdFontSmall,DRAWING_X_OFFSET+DRAWING_WIDTH+115,DRAWING_Y_OFFSET-DRAWING_HEIGHT+((string_height+5)*2),temp_buffer,color_black);
+	gdImageString(histogram_image,gdFontSmall,DRAWING_X_OFFSET+DRAWING_WIDTH+115,DRAWING_Y_OFFSET-DRAWING_HEIGHT+((string_height+5)*2),(unsigned char *)temp_buffer,color_black);
 
 	snprintf(temp_buffer,sizeof(temp_buffer)-1,"%s:",(display_type==DISPLAY_SERVICE_HISTOGRAM)?"Unknown":"Unreachable");
 	temp_buffer[sizeof(temp_buffer)-1]='\x0';
 	string_width=gdFontSmall->w*strlen(temp_buffer);
-	gdImageString(histogram_image,gdFontSmall,DRAWING_X_OFFSET+DRAWING_WIDTH+15,DRAWING_Y_OFFSET-DRAWING_HEIGHT+((string_height+5)*3),temp_buffer,(display_type==DISPLAY_SERVICE_HISTOGRAM)?color_orange:color_darkred);
+	gdImageString(histogram_image,gdFontSmall,DRAWING_X_OFFSET+DRAWING_WIDTH+15,DRAWING_Y_OFFSET-DRAWING_HEIGHT+((string_height+5)*3),(unsigned char *)temp_buffer,(display_type==DISPLAY_SERVICE_HISTOGRAM)?color_orange:color_darkred);
 
 	state3_avg=(double)((double)state3_sum/(double)total_buckets);
 	snprintf(temp_buffer,sizeof(temp_buffer)-1,"%5lu %5lu %5lu   %.2f",state3_min,state3_max,state3_sum,state3_avg);
 	temp_buffer[sizeof(temp_buffer)-1]='\x0';
 	string_width=gdFontSmall->w*strlen(temp_buffer);
-	gdImageString(histogram_image,gdFontSmall,DRAWING_X_OFFSET+DRAWING_WIDTH+115,DRAWING_Y_OFFSET-DRAWING_HEIGHT+((string_height+5)*3),temp_buffer,color_black);
+	gdImageString(histogram_image,gdFontSmall,DRAWING_X_OFFSET+DRAWING_WIDTH+115,DRAWING_Y_OFFSET-DRAWING_HEIGHT+((string_height+5)*3),(unsigned char *)temp_buffer,color_black);
 
 	if(display_type==DISPLAY_SERVICE_HISTOGRAM){
 
 		snprintf(temp_buffer,sizeof(temp_buffer)-1,"Critical:");
 		temp_buffer[sizeof(temp_buffer)-1]='\x0';
 		string_width=gdFontSmall->w*strlen(temp_buffer);
-		gdImageString(histogram_image,gdFontSmall,DRAWING_X_OFFSET+DRAWING_WIDTH+15,DRAWING_Y_OFFSET-DRAWING_HEIGHT+((string_height+5)*4),temp_buffer,color_red);
+		gdImageString(histogram_image,gdFontSmall,DRAWING_X_OFFSET+DRAWING_WIDTH+15,DRAWING_Y_OFFSET-DRAWING_HEIGHT+((string_height+5)*4),(unsigned char *)temp_buffer,color_red);
 
 		state4_avg=(double)((double)state4_sum/(double)total_buckets);
 		snprintf(temp_buffer,sizeof(temp_buffer)-1,"%5lu %5lu %5lu   %.2f",state4_min,state4_max,state4_sum,state4_avg);
 		temp_buffer[sizeof(temp_buffer)-1]='\x0';
 		string_width=gdFontSmall->w*strlen(temp_buffer);
-		gdImageString(histogram_image,gdFontSmall,DRAWING_X_OFFSET+DRAWING_WIDTH+115,DRAWING_Y_OFFSET-DRAWING_HEIGHT+((string_height+5)*4),temp_buffer,color_black);
+		gdImageString(histogram_image,gdFontSmall,DRAWING_X_OFFSET+DRAWING_WIDTH+115,DRAWING_Y_OFFSET-DRAWING_HEIGHT+((string_height+5)*4),(unsigned char *)temp_buffer,color_black);
 	        }
 
 	return;
@@ -2177,13 +2191,13 @@ void read_archived_state_data(void){
 
 /* grabs archives state data from a log file */
 void scan_log_file_for_archived_state_data(char *filename){
-	char input_buffer[MAX_INPUT_BUFFER];
-	char input_buffer2[MAX_INPUT_BUFFER];
+	char *input=NULL;
+	char *input2=NULL;
 	char entry_host_name[MAX_INPUT_BUFFER];
 	char entry_svc_description[MAX_INPUT_BUFFER];
 	char *temp_buffer;
 	time_t time_stamp;
-	FILE *fp;
+	mmapfile *thefile;
 
 	/* print something so browser doesn't time out */
 	if(mode==CREATE_HTML){
@@ -2191,8 +2205,7 @@ void scan_log_file_for_archived_state_data(char *filename){
 		fflush(NULL);
 	        }
 
-	fp=fopen(filename,"r");
-	if(fp==NULL){
+	if((thefile=mmap_fopen(filename))==NULL){
 #ifdef DEBUG2
 		printf("Could not open file '%s' for reading.\n",filename);
 #endif
@@ -2203,37 +2216,42 @@ void scan_log_file_for_archived_state_data(char *filename){
 	printf("Scanning log file '%s' for archived state data...\n",filename);
 #endif
 
-	while(read_line(input_buffer,MAX_INPUT_BUFFER,fp)){
+	while(1){
 
-	        if(feof(fp))
-		        break;
+		/* free memory */
+		free(input);
+		free(input2);
+		input=NULL;
+		input2=NULL;
 
-		if(input_buffer==NULL)
-		        continue;
+		/* read the next line */
+		if((input=mmap_fgets(thefile))==NULL)
+			break;
 
-		strcpy(input_buffer2,input_buffer);
-		temp_buffer=strtok(input_buffer2,"]");
+		strip(input);
+
+		if((input2=strdup(input))==NULL)
+			continue;
+
+		temp_buffer=my_strtok(input2,"]");
 		time_stamp=(temp_buffer==NULL)?(time_t)0:(time_t)strtoul(temp_buffer+1,NULL,10);
 
 		/* program starts/restarts */
-		if(strstr(input_buffer," starting..."))
+		if(strstr(input," starting..."))
 			add_archived_state(AS_PROGRAM_START,time_stamp);
-		if(strstr(input_buffer," restarting..."))
+		if(strstr(input," restarting..."))
 			add_archived_state(AS_PROGRAM_START,time_stamp);
 
 		/* program stops */
-		if(strstr(input_buffer," shutting down..."))
+		if(strstr(input," shutting down..."))
 			add_archived_state(AS_PROGRAM_END,time_stamp);
-		if(strstr(input_buffer,"Bailing out"))
+		if(strstr(input,"Bailing out"))
 			add_archived_state(AS_PROGRAM_END,time_stamp);
 
 		if(display_type==DISPLAY_HOST_HISTOGRAM){
-			if(strstr(input_buffer,"HOST ALERT:")){
-
-				strcpy(input_buffer2,input_buffer);
+			if(strstr(input,"HOST ALERT:")){
 
 				/* get host name */
-				temp_buffer=my_strtok(input_buffer2,"]");
 				temp_buffer=my_strtok(NULL,":");
 				temp_buffer=my_strtok(NULL,";");
 				strncpy(entry_host_name,(temp_buffer==NULL)?"":temp_buffer+1,sizeof(entry_host_name));
@@ -2243,28 +2261,25 @@ void scan_log_file_for_archived_state_data(char *filename){
 					continue;
 
 				/* skip soft states if necessary */
-				if(!(graph_statetypes & GRAPH_SOFT_STATETYPES) && strstr(input_buffer,";SOFT;"))
+				if(!(graph_statetypes & GRAPH_SOFT_STATETYPES) && strstr(input,";SOFT;"))
 					continue;
 
 				/* skip hard states if necessary */
-				if(!(graph_statetypes & GRAPH_HARD_STATETYPES) && strstr(input_buffer,";HARD;"))
+				if(!(graph_statetypes & GRAPH_HARD_STATETYPES) && strstr(input,";HARD;"))
 					continue;
 
-				if(strstr(input_buffer,";DOWN;"))
+				if(strstr(input,";DOWN;"))
 					add_archived_state(AS_HOST_DOWN,time_stamp);
-				else if(strstr(input_buffer,";UNREACHABLE;"))
+				else if(strstr(input,";UNREACHABLE;"))
 					add_archived_state(AS_HOST_UNREACHABLE,time_stamp);
-				else if(strstr(input_buffer,";RECOVERY") || strstr(input_buffer,";UP;"))
+				else if(strstr(input,";RECOVERY") || strstr(input,";UP;"))
 					add_archived_state(AS_HOST_UP,time_stamp);
 			        }
 		        }
 		if(display_type==DISPLAY_SERVICE_HISTOGRAM){
-			if(strstr(input_buffer,"SERVICE ALERT:")){
-
-				strcpy(input_buffer2,input_buffer);
+			if(strstr(input,"SERVICE ALERT:")){
 
 				/* get host name */
-				temp_buffer=my_strtok(input_buffer2,"]");
 				temp_buffer=my_strtok(NULL,":");
 				temp_buffer=my_strtok(NULL,";");
 				strncpy(entry_host_name,(temp_buffer==NULL)?"":temp_buffer+1,sizeof(entry_host_name));
@@ -2282,27 +2297,30 @@ void scan_log_file_for_archived_state_data(char *filename){
 					continue;
 
 				/* skip soft states if necessary */
-				if(!(graph_statetypes & GRAPH_SOFT_STATETYPES) && strstr(input_buffer,";SOFT;"))
+				if(!(graph_statetypes & GRAPH_SOFT_STATETYPES) && strstr(input,";SOFT;"))
 					continue;
 
 				/* skip hard states if necessary */
-				if(!(graph_statetypes & GRAPH_HARD_STATETYPES) && strstr(input_buffer,";HARD;"))
+				if(!(graph_statetypes & GRAPH_HARD_STATETYPES) && strstr(input,";HARD;"))
 					continue;
 
-				if(strstr(input_buffer,";CRITICAL;"))
+				if(strstr(input,";CRITICAL;"))
 					add_archived_state(AS_SVC_CRITICAL,time_stamp);
-				else if(strstr(input_buffer,";WARNING;"))
+				else if(strstr(input,";WARNING;"))
 					add_archived_state(AS_SVC_WARNING,time_stamp);
-				else if(strstr(input_buffer,";UNKNOWN;"))
+				else if(strstr(input,";UNKNOWN;"))
 					add_archived_state(AS_SVC_UNKNOWN,time_stamp);
-				else if(strstr(input_buffer,";RECOVERY;") || strstr(input_buffer,";OK;"))
+				else if(strstr(input,";RECOVERY;") || strstr(input,";OK;"))
 					add_archived_state(AS_SVC_OK,time_stamp);
 			        }
 		        }
 		
 	        }
 
-	fclose(fp);
+	/* free memory and close the file */
+	free(input);
+	free(input2);
+	mmap_fclose(thefile);
 	
 	return;
         }
@@ -2381,6 +2399,10 @@ void convert_timeperiod_to_times(int type){
 		t2=current_time;
 		t1=current_time-(7*24*60*60);
 		break;
+	case TIMEPERIOD_LAST31DAYS:
+		t2=current_time;
+		t1=current_time-(31*24*60*60);
+		break;
 	default:
 		break;
 	        }
