@@ -2,17 +2,17 @@
  *
  * NOTIFICATIONS.C - Nagios Notifications CGI
  *
- * Copyright (c) 1999-2002 Ethan Galstad (nagios@nagios.org)
- * Last Modified: 08-12-2002
+ * Copyright (c) 1999-2004 Ethan Galstad (nagios@nagios.org)
+ * Last Modified: 10-30-2004
  *
  * This CGI program will display the notification events for 
  * a given host or contact or for all contacts/hosts.
  *
+ * License:
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU General Public License version 2 as
+ * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -24,14 +24,12 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  ***********************************************************************/
 
-#include "../common/config.h"
-#include "../common/locations.h"
-#include "../common/common.h"
+#include "../include/config.h"
+#include "../include/common.h"
 
-#include "getcgi.h"
-#include "cgiutils.h"
-#include "auth.h"
-#include "lifo.h"
+#include "../include/getcgi.h"
+#include "../include/cgiutils.h"
+#include "../include/cgiauth.h"
 
 extern char main_config_file[MAX_FILENAME_LENGTH];
 extern char url_html_path[MAX_FILENAME_LENGTH];
@@ -91,10 +89,10 @@ int main(void){
 	reset_cgi_vars();
 
 	/* read the CGI configuration file */
-	result=read_cgi_config_file(DEFAULT_CGI_CONFIG_FILE);
+	result=read_cgi_config_file(get_cgi_config_location());
 	if(result==ERROR){
 		document_header(FALSE);
-		cgi_config_file_error(DEFAULT_CGI_CONFIG_FILE);
+		cgi_config_file_error(get_cgi_config_location());
 		document_footer();
 		return ERROR;
 	        }
@@ -238,11 +236,13 @@ int main(void){
 		printf("<option value=%d %s>Service unknown\n",NOTIFICATION_SERVICE_UNKNOWN,(notification_options==NOTIFICATION_SERVICE_UNKNOWN)?"selected":"");
 		printf("<option value=%d %s>Service critical\n",NOTIFICATION_SERVICE_CRITICAL,(notification_options==NOTIFICATION_SERVICE_CRITICAL)?"selected":"");
 		printf("<option value=%d %s>Service recovery\n",NOTIFICATION_SERVICE_RECOVERY,(notification_options==NOTIFICATION_SERVICE_RECOVERY)?"selected":"");
+		printf("<option value=%d %s>Service flapping\n",NOTIFICATION_SERVICE_FLAP,(notification_options==NOTIFICATION_SERVICE_FLAP)?"selected":"");
 		if(query_type!=FIND_SERVICE){
 			printf("<option value=%d %s>Host acknowledgements\n",NOTIFICATION_HOST_ACK,(notification_options==NOTIFICATION_HOST_ACK)?"selected":"");
 			printf("<option value=%d %s>Host down\n",NOTIFICATION_HOST_DOWN,(notification_options==NOTIFICATION_HOST_DOWN)?"selected":"");
 			printf("<option value=%d %s>Host unreachable\n",NOTIFICATION_HOST_UNREACHABLE,(notification_options==NOTIFICATION_HOST_UNREACHABLE)?"selected":"");
 			printf("<option value=%d %s>Host recovery\n",NOTIFICATION_HOST_RECOVERY,(notification_options==NOTIFICATION_HOST_RECOVERY)?"selected":"");
+			printf("<option value=%d %s>Host flapping\n",NOTIFICATION_HOST_FLAP,(notification_options==NOTIFICATION_HOST_FLAP)?"selected":"");
 	                }
 		printf("</select></td>\n");
 		printf("</tr>\n");
@@ -312,9 +312,11 @@ void document_header(int use_stylesheet){
 	printf("Alert Notifications\n");
 	printf("</title>\n");
 
-	if(use_stylesheet==TRUE)
+	if(use_stylesheet==TRUE){
+		printf("<LINK REL='stylesheet' TYPE='text/css' HREF='%s%s'>\n",url_stylesheets_path,COMMON_CSS);
 		printf("<LINK REL='stylesheet' TYPE='text/css' HREF='%s%s'>\n",url_stylesheets_path,NOTIFICATIONS_CSS);
-
+	        }
+	
 	printf("</head>\n");
 
 	printf("<body CLASS='notifications'>\n");
@@ -455,8 +457,8 @@ int process_cgivars(void){
 
 
 void display_notifications(void){
-	FILE *fp=NULL;
-	char input_buffer[MAX_INPUT_BUFFER];
+	mmapfile *thefile;
+	char *input=NULL;
 	char *temp_buffer;
 	char date_time[MAX_DATETIME_LENGTH];
 	char alert_level[MAX_INPUT_BUFFER];
@@ -491,8 +493,7 @@ void display_notifications(void){
 
 	if(use_lifo==FALSE){
 
-		fp=fopen(log_file_to_use,"r");
-		if(fp==NULL){
+		if((thefile=mmap_fopen(log_file_to_use))==NULL){
 			printf("<P><DIV CLASS='errorMessage'>Error: Cannot open log file '%s' for reading!</DIV></P>",log_file_to_use);
 			return;
 		        }
@@ -515,27 +516,30 @@ void display_notifications(void){
 	total_notifications=0;
   
 	while(1){
+
+		free(input);
     
 		if(use_lifo==TRUE){
-			if(pop_lifo(input_buffer,(int)sizeof(input_buffer))!=LIFO_OK)
+			if((input=pop_lifo())==NULL)
 				break;
 		        }
 		else{
-			fgets(input_buffer,(int)(sizeof(input_buffer)-1),fp);
-			if(feof(fp))
+			if((input=mmap_fgets(thefile))==NULL)
 				break;
 		        }
 
-		/* see if this line contains the notification event string */
-		if(strstr(input_buffer,HOST_NOTIFICATION_STRING)||strstr(input_buffer,SERVICE_NOTIFICATION_STRING)){
+		strip(input);
 
-			if(strstr(input_buffer,HOST_NOTIFICATION_STRING))
+		/* see if this line contains the notification event string */
+		if(strstr(input,HOST_NOTIFICATION_STRING)||strstr(input,SERVICE_NOTIFICATION_STRING)){
+
+			if(strstr(input,HOST_NOTIFICATION_STRING))
 				notification_type=HOST_NOTIFICATION;
 			else
 				notification_type=SERVICE_NOTIFICATION;
       
 			/* get the date/time */
-			temp_buffer=(char *)strtok(input_buffer,"]");
+			temp_buffer=(char *)strtok(input,"]");
 			t=(time_t)(temp_buffer==NULL)?0L:strtoul(temp_buffer+1,NULL,10);
 			get_time_string(&t,date_time,(int)sizeof(date_time),SHORT_DATE_TIME);
 			strip(date_time);
@@ -582,6 +586,16 @@ void display_notifications(void){
 					notification_detail_type=NOTIFICATION_SERVICE_ACK;
 					strcpy(alert_level_class,"ACKNOWLEDGEMENT");
 				        }
+				else if(strstr(alert_level,"FLAPPINGSTART (")){
+					strcpy(alert_level,"FLAPPING START");
+					notification_detail_type=NOTIFICATION_SERVICE_FLAP;
+					strcpy(alert_level_class,"UNKNOWN");
+				        }
+				else if(strstr(alert_level,"FLAPPINGSTOP (")){
+					strcpy(alert_level,"FLAPPING STOP");
+					notification_detail_type=NOTIFICATION_SERVICE_FLAP;
+					strcpy(alert_level_class,"UNKNOWN");
+				        }
 				else{
 					strcpy(alert_level,"UNKNOWN");
 					notification_detail_type=NOTIFICATION_SERVICE_UNKNOWN;
@@ -610,6 +624,16 @@ void display_notifications(void){
 					strcpy(alert_level_class,"HOSTACKNOWLEDGEMENT");
 					notification_detail_type=NOTIFICATION_HOST_ACK;
 				        }
+				else if(strstr(alert_level,"FLAPPINGSTART (")){
+					strcpy(alert_level,"FLAPPING START");
+					strcpy(alert_level_class,"UNKNOWN");
+					notification_detail_type=NOTIFICATION_HOST_FLAP;
+				        }
+				else if(strstr(alert_level,"FLAPPINGSTOP (")){
+					strcpy(alert_level,"FLAPPING STOP");
+					strcpy(alert_level_class,"UNKNOWN");
+					notification_detail_type=NOTIFICATION_HOST_FLAP;
+				        }
 			        }
 
 			/* get the method name */
@@ -657,12 +681,12 @@ void display_notifications(void){
 
 			/* make sure user has authorization to view this notification */
 			if(notification_type==HOST_NOTIFICATION){
-				temp_host=find_host(host_name,NULL);
+				temp_host=find_host(host_name);
 				if(is_authorized_for_host(temp_host,&current_authdata)==FALSE)
 					show_entry=FALSE;
 			        }
 			else{
-				temp_service=find_service(host_name,service_name,NULL);
+				temp_service=find_service(host_name,service_name);
 				if(is_authorized_for_service(temp_service,&current_authdata)==FALSE)
 					show_entry=FALSE;
 			        }
@@ -715,10 +739,12 @@ void display_notifications(void){
 		printf(" in %s log file</DIV></P>",(log_archive==0)?"the current":"this archived");
 	        }
 
+	free(input);
+
 	if(use_lifo==TRUE)
 		free_lifo_memory();
 	else
-		fclose(fp);
+		mmap_fclose(thefile);
 
 	return;
         }
