@@ -2,15 +2,14 @@
  *
  * SUMMARY.C -  Nagios Alert Summary CGI
  *
- * Copyright (c) 2002-2003 Ethan Galstad (nagios@nagios.org)
- * Last Modified: 01-07-2003
+ * Copyright (c) 2002-2005 Ethan Galstad (nagios@nagios.org)
+ * Last Modified: 05-22-2005
  *
  * License:
  * 
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU General Public License version 2 as
+ * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -22,26 +21,25 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *************************************************************************/
 
-#include "../common/config.h"
-#include "../common/locations.h"
-#include "../common/common.h"
-#include "../common/objects.h"
-#include "../common/comments.h"
-#include "../common/statusdata.h"
+#include "../include/config.h"
+#include "../include/common.h"
+#include "../include/objects.h"
+#include "../include/comments.h"
+#include "../include/statusdata.h"
 
-#include "cgiutils.h"
-#include "getcgi.h"
-#include "auth.h"
-#include "edata.h"
+#include "../include/cgiutils.h"
+#include "../include/getcgi.h"
+#include "../include/cgiauth.h"
 
 
 extern char main_config_file[MAX_FILENAME_LENGTH];
 extern char url_images_path[MAX_FILENAME_LENGTH];
 extern char url_stylesheets_path[MAX_FILENAME_LENGTH];
 
-extern hostgroup *hostgroup_list;
 extern host *host_list;
+extern hostgroup *hostgroup_list;
 extern service *service_list;
+extern servicegroup *servicegroup_list;
 
 extern int       log_rotation_method;
 
@@ -58,6 +56,7 @@ extern int       log_rotation_method;
 #define REPORT_HOSTGROUP_ALERT_TOTALS  4 
 #define REPORT_HOST_ALERT_TOTALS       5
 #define REPORT_SERVICE_ALERT_TOTALS    6
+#define REPORT_SERVICEGROUP_ALERT_TOTALS 7
 
 /* standard report types */
 #define SREPORT_NONE                   0
@@ -81,6 +80,7 @@ extern int       log_rotation_method;
 #define TIMEPERIOD_LASTYEAR	10
 #define TIMEPERIOD_LAST24HOURS	11
 #define TIMEPERIOD_LAST7DAYS	12
+#define TIMEPERIOD_LAST31DAYS	13
 
 #define AE_SOFT_STATE           1
 #define AE_HARD_STATE           2
@@ -136,6 +136,8 @@ void display_recent_alerts(void);
 void display_alert_totals(void);
 void display_hostgroup_alert_totals(void);
 void display_specific_hostgroup_alert_totals(hostgroup *);
+void display_servicegroup_alert_totals(void);
+void display_specific_servicegroup_alert_totals(servicegroup *);
 void display_host_alert_totals(void);
 void display_specific_host_alert_totals(host *);
 void display_service_alert_totals(void);
@@ -177,11 +179,14 @@ int host_states=AE_HOST_UP+AE_HOST_DOWN+AE_HOST_UNREACHABLE;
 int service_states=AE_SERVICE_OK+AE_SERVICE_WARNING+AE_SERVICE_UNKNOWN+AE_SERVICE_CRITICAL;
 
 int show_all_hostgroups=TRUE;
+int show_all_servicegroups=TRUE;
 int show_all_hosts=TRUE;
 
 char *target_hostgroup_name="";
+char *target_servicegroup_name="";
 char *target_host_name="";
 hostgroup *target_hostgroup=NULL;
+servicegroup *target_servicegroup=NULL;
 host *target_host=NULL;
 
 int earliest_archive=0;
@@ -204,12 +209,9 @@ int main(int argc, char **argv){
 	char start_timestring[MAX_DATETIME_LENGTH];
 	char end_timestring[MAX_DATETIME_LENGTH];
 	host *temp_host;
-	service *temp_service;
-	int is_authorized=TRUE;
-	time_t report_start_time;
-	time_t report_end_time;
 	int days, hours, minutes, seconds;
 	hostgroup *temp_hostgroup;
+	servicegroup *temp_servicegroup;
 	time_t t3;
 	time_t current_time;
 	struct tm *t;
@@ -219,10 +221,10 @@ int main(int argc, char **argv){
 	reset_cgi_vars();
 	
 	/* read the CGI configuration file */
-	result=read_cgi_config_file(DEFAULT_CGI_CONFIG_FILE);
+	result=read_cgi_config_file(get_cgi_config_location());
 	if(result==ERROR){
 		document_header(FALSE);
-		cgi_config_file_error(DEFAULT_CGI_CONFIG_FILE);
+		cgi_config_file_error(get_cgi_config_location());
 		document_footer();
 		return ERROR;
 	        }
@@ -293,13 +295,17 @@ int main(int argc, char **argv){
 			printf("<DIV ALIGN=CENTER CLASS='dataTitle'>\n");
 			if(display_type==REPORT_TOP_ALERTS)
 				printf("Top Alert Producers");
-			else if(display_type==REPORT_ALERT_TOTALS || display_type==REPORT_HOSTGROUP_ALERT_TOTALS || display_type==REPORT_HOST_ALERT_TOTALS || display_type==REPORT_SERVICE_ALERT_TOTALS)
+			else if(display_type==REPORT_ALERT_TOTALS || display_type==REPORT_HOSTGROUP_ALERT_TOTALS || display_type==REPORT_SERVICEGROUP_ALERT_TOTALS || display_type==REPORT_HOST_ALERT_TOTALS || display_type==REPORT_SERVICE_ALERT_TOTALS)
 				printf("Alert Totals");
 			else
 				printf("Most Recent Alerts");
 
 			if(show_all_hostgroups==FALSE)
 				printf(" For Hostgroup '%s'",target_hostgroup_name);
+			else if(show_all_servicegroups==FALSE)
+				printf(" For Servicegroup '%s'",target_servicegroup_name);
+			else if(show_all_hosts==FALSE)
+				printf(" For Host '%s'",target_host_name);
 
 			printf("</DIV>\n");
 
@@ -388,16 +394,6 @@ int main(int argc, char **argv){
 			printf("</td>\n");
 			printf("</tr>\n");
 
-			printf("<tr>\n");
-			printf("<td valign=top align=left class='optBoxItem'>Hostgroup:</td>\n");
-			printf("<td valign=top align=left class='optBoxValue'>\n");
-			if(show_all_hostgroups==TRUE)
-				printf("All Hostgroups");
-			else
-				printf("%s",target_hostgroup_name);
-			printf("</td>\n");
-			printf("</tr>\n");
-
 			printf("<tr>\n");
 			printf("<td valign=top align=left colspan=2 class='optBoxItem'>\n");
 			printf("<form action='%s' method='GET'>\n",SUMMARY_CGI);
@@ -418,6 +414,8 @@ int main(int argc, char **argv){
 				display_context_help(CONTEXTHELP_SUMMARY_HOST_ALERT_TOTALS);
 			else if(display_type==REPORT_SERVICE_ALERT_TOTALS)
 				display_context_help(CONTEXTHELP_SUMMARY_SERVICE_ALERT_TOTALS);
+			else if(display_type==REPORT_SERVICEGROUP_ALERT_TOTALS)
+				display_context_help(CONTEXTHELP_SUMMARY_SERVICEGROUP_ALERT_TOTALS);
 			else
 				display_context_help(CONTEXTHELP_SUMMARY_RECENT_ALERTS);
 			printf("</td></tr>\n");
@@ -428,11 +426,9 @@ int main(int argc, char **argv){
 		else{
 			printf("<table border=0>\n");
 
-#ifdef CONTEXT_HELP
 			printf("<tr><td></td><td align=right valign=bottom>\n");
 			display_context_help(CONTEXTHELP_SUMMARY_MENU);
 			printf("</td></tr>\n");
-#endif
 
 			printf("</table>\n");
 		        }
@@ -506,6 +502,7 @@ int main(int argc, char **argv){
 		printf("<option value=%d>Alert Totals\n",REPORT_ALERT_TOTALS);
 		printf("<option value=%d>Alert Totals By Hostgroup\n",REPORT_HOSTGROUP_ALERT_TOTALS);
 		printf("<option value=%d>Alert Totals By Host\n",REPORT_HOST_ALERT_TOTALS);
+		printf("<option value=%d>Alert Totals By Servicegroup\n",REPORT_SERVICEGROUP_ALERT_TOTALS);
 		printf("<option value=%d>Alert Totals By Service\n",REPORT_SERVICE_ALERT_TOTALS);
 		printf("<option value=%d>Top Alert Producers\n",REPORT_TOP_ALERTS);
 		printf("</select>\n");
@@ -515,13 +512,14 @@ int main(int argc, char **argv){
 		printf("<td valign=top class='reportSelectSubTitle'>Report Period:</td>\n");
 		printf("<td valign=top align=left class='optBoxItem'>\n");
 		printf("<select name='timeperiod'>\n");
-		printf("<option value=last24hours>Last 24 Hours\n");
 		printf("<option value=today>Today\n");
+		printf("<option value=last24hours>Last 24 Hours\n");
 		printf("<option value=yesterday>Yesterday\n");
 		printf("<option value=thisweek>This Week\n");
 		printf("<option value=last7days SELECTED>Last 7 Days\n");
 		printf("<option value=lastweek>Last Week\n");
 		printf("<option value=thismonth>This Month\n");
+		printf("<option value=last31days>Last 31 Days\n");
 		printf("<option value=lastmonth>Last Month\n");
 		printf("<option value=thisyear>This Year\n");
 		printf("<option value=lastyear>Last Year\n");
@@ -530,7 +528,7 @@ int main(int argc, char **argv){
 		printf("</td>\n");
 		printf("</tr>\n");
 
-		printf("<tr><td colspan=2 valign=top calss='reportSelectSubTitle'><i>If Custom Report Period...</i></td></tr>\n");
+		printf("<tr><td valign=top class='reportSelectSubTitle'>If Custom Report Period...</td></tr>\n");
 
 		printf("<tr>");
 		printf("<td valign=top class='reportSelectSubTitle'>Start Date (Inclusive):</td>\n");
@@ -594,9 +592,20 @@ int main(int argc, char **argv){
 		printf("</select>\n");
 		printf("</td></tr>\n");
 
+		printf("<tr><td class='reportSelectSubTitle' valign=center>Limit To Servicegroup:</td><td align=left valign=center class='reportSelectItem'>\n");
+		printf("<select name='servicegroup'>\n");
+		printf("<option value='all'>** ALL SERVICEGROUPS **\n");
+		for(temp_servicegroup=servicegroup_list;temp_servicegroup!=NULL;temp_servicegroup=temp_servicegroup->next){
+			if(is_authorized_for_servicegroup(temp_servicegroup,&current_authdata)==TRUE)
+				printf("<option value='%s'>%s\n",temp_servicegroup->group_name,temp_servicegroup->group_name);
+		        }
+		printf("</select>\n");
+		printf("</td></tr>\n");
+
 		printf("<tr><td class='reportSelectSubTitle' valign=center>Limit To Host:</td><td align=left valign=center class='reportSelectItem'>\n");
 		printf("<select name='host'>\n");
 		printf("<option value='all'>** ALL HOSTS **\n");
+
 		for(temp_host=host_list;temp_host!=NULL;temp_host=temp_host->next){
 			if(is_authorized_for_host(temp_host,&current_authdata)==TRUE)
 				printf("<option value='%s'>%s\n",temp_host->name,temp_host->name);
@@ -703,9 +712,11 @@ void document_header(int use_stylesheet){
 	printf("Nagios Event Summary\n");
 	printf("</title>\n");
 
-	if(use_stylesheet==TRUE)
+	if(use_stylesheet==TRUE){
+		printf("<LINK REL='stylesheet' TYPE='text/css' HREF='%s%s'>\n",url_stylesheets_path,COMMON_CSS);
 		printf("<LINK REL='stylesheet' TYPE='text/css' HREF='%s%s'>\n",url_stylesheets_path,SUMMARY_CSS);
-
+	        }
+	
 	printf("</head>\n");
 
 	printf("<BODY CLASS='summary'>\n");
@@ -810,6 +821,8 @@ int process_cgivars(void){
 				timeperiod_type=TIMEPERIOD_LAST24HOURS;
 			else if(!strcmp(variables[x],"last7days"))
 				timeperiod_type=TIMEPERIOD_LAST7DAYS;
+			else if(!strcmp(variables[x],"last31days"))
+				timeperiod_type=TIMEPERIOD_LAST31DAYS;
 			else if(!strcmp(variables[x],"custom"))
 				timeperiod_type=TIMEPERIOD_CUSTOM;
 			else
@@ -1127,7 +1140,29 @@ int process_cgivars(void){
 				show_all_hostgroups=TRUE;
 			else{
 				show_all_hostgroups=FALSE;
-				target_hostgroup=find_hostgroup(target_hostgroup_name,NULL);
+				target_hostgroup=find_hostgroup(target_hostgroup_name);
+			        }
+		        }
+
+		/* we found the servicegroup argument */
+		else if(!strcmp(variables[x],"servicegroup")){
+			x++;
+			if(variables[x]==NULL){
+				error=TRUE;
+				break;
+			        }
+
+			target_servicegroup_name=(char *)malloc(strlen(variables[x])+1);
+			if(target_servicegroup_name==NULL)
+				target_servicegroup_name="";
+			else
+				strcpy(target_servicegroup_name,variables[x]);
+
+			if(!strcmp(target_servicegroup_name,"all"))
+				show_all_servicegroups=TRUE;
+			else{
+				show_all_servicegroups=FALSE;
+				target_servicegroup=find_servicegroup(target_servicegroup_name);
 			        }
 		        }
 
@@ -1149,7 +1184,7 @@ int process_cgivars(void){
 				show_all_hosts=TRUE;
 			else{
 				show_all_hosts=FALSE;
-				target_host=find_host(target_host_name,NULL);
+				target_host=find_host(target_host_name);
 			        }
 		        }
 	        }
@@ -1195,8 +1230,8 @@ void read_archived_event_data(void){
 
 /* grabs archived event data from a log file */
 void scan_log_file_for_archived_event_data(char *filename){
-	char input_buffer[MAX_INPUT_BUFFER];
-	char input_buffer2[MAX_INPUT_BUFFER];
+	char *input=NULL;
+	char *input2=NULL;
 	char entry_host_name[MAX_INPUT_BUFFER];
 	char entry_svc_description[MAX_INPUT_BUFFER];
 	int state;
@@ -1204,39 +1239,46 @@ void scan_log_file_for_archived_event_data(char *filename){
 	char *temp_buffer;
 	char *plugin_output;
 	time_t time_stamp;
-	FILE *fp;
+	mmapfile *thefile;
 
-	fp=fopen(filename,"r");
-	if(fp==NULL)
+
+	if((thefile=mmap_fopen(filename))==NULL)
 		return;
 
-	while(read_line(input_buffer,MAX_INPUT_BUFFER,fp)){
+	while(1){
 
-		if(input_buffer==NULL)
-		        continue;
+		/* free memory */
+		free(input);
+		free(input2);
+		input=NULL;
+		input2=NULL;
 
-		strcpy(input_buffer2,input_buffer);
+		/* read the next line */
+		if((input=mmap_fgets(thefile))==NULL)
+			break;
+
+		strip(input);
+
+		if((input2=strdup(input))==NULL)
+			continue;
 
 		/* get the timestamp */
-		temp_buffer=strtok(input_buffer2,"]");
+		temp_buffer=my_strtok(input2,"]");
 		time_stamp=(temp_buffer==NULL)?(time_t)0:(time_t)strtoul(temp_buffer+1,NULL,10);
 		if(time_stamp<t1 || time_stamp>t2)
 			continue;
 		
 		/* host alerts */
-		if(strstr(input_buffer,"HOST ALERT:")){
-
-			strcpy(input_buffer2,input_buffer);
+		if(strstr(input,"HOST ALERT:")){
 
 			/* get host name */
-			temp_buffer=my_strtok(input_buffer2,"]");
 			temp_buffer=my_strtok(NULL,":");
 			temp_buffer=my_strtok(NULL,";");
 			strncpy(entry_host_name,(temp_buffer==NULL)?"":temp_buffer+1,sizeof(entry_host_name));
 			entry_host_name[sizeof(entry_host_name)-1]='\x0';
 
 			/* state type */
-			if(strstr(input_buffer,";SOFT;"))
+			if(strstr(input,";SOFT;"))
 				state_type=AE_SOFT_STATE;
 			else
 				state_type=AE_HARD_STATE;
@@ -1248,11 +1290,11 @@ void scan_log_file_for_archived_event_data(char *filename){
 			plugin_output=my_strtok(NULL,"\n");
 
 			/* state */
-			if(strstr(input_buffer,";DOWN;"))
+			if(strstr(input,";DOWN;"))
 				state=AE_HOST_DOWN;
-			else if(strstr(input_buffer,";UNREACHABLE;"))
+			else if(strstr(input,";UNREACHABLE;"))
 				state=AE_HOST_UNREACHABLE;
-			else if(strstr(input_buffer,";RECOVERY") || strstr(input_buffer,";UP;"))
+			else if(strstr(input,";RECOVERY") || strstr(input,";UP;"))
 				state=AE_HOST_UP;
 			else
 				continue;
@@ -1261,12 +1303,9 @@ void scan_log_file_for_archived_event_data(char *filename){
 		        }
 
 		/* service alerts */
-		if(strstr(input_buffer,"SERVICE ALERT:")){
-
-			strcpy(input_buffer2,input_buffer);
+		if(strstr(input,"SERVICE ALERT:")){
 
 			/* get host name */
-			temp_buffer=my_strtok(input_buffer2,"]");
 			temp_buffer=my_strtok(NULL,":");
 			temp_buffer=my_strtok(NULL,";");
 			strncpy(entry_host_name,(temp_buffer==NULL)?"":temp_buffer+1,sizeof(entry_host_name));
@@ -1278,7 +1317,7 @@ void scan_log_file_for_archived_event_data(char *filename){
 			entry_svc_description[sizeof(entry_svc_description)-1]='\x0';
 
 			/* state type */
-			if(strstr(input_buffer,";SOFT;"))
+			if(strstr(input,";SOFT;"))
 				state_type=AE_SOFT_STATE;
 			else
 				state_type=AE_HARD_STATE;
@@ -1290,13 +1329,13 @@ void scan_log_file_for_archived_event_data(char *filename){
 			plugin_output=my_strtok(NULL,"\n");
 
 			/* state */
-			if(strstr(input_buffer,";WARNING;"))
+			if(strstr(input,";WARNING;"))
 				state=AE_SERVICE_WARNING;
-			else if(strstr(input_buffer,";UNKNOWN;"))
+			else if(strstr(input,";UNKNOWN;"))
 				state=AE_SERVICE_UNKNOWN;
-			else if(strstr(input_buffer,";CRITICAL;"))
+			else if(strstr(input,";CRITICAL;"))
 				state=AE_SERVICE_CRITICAL;
-			else if(strstr(input_buffer,";RECOVERY") || strstr(input_buffer,";OK;"))
+			else if(strstr(input,";RECOVERY") || strstr(input,";OK;"))
 				state=AE_SERVICE_OK;
 			else
 				continue;
@@ -1305,7 +1344,10 @@ void scan_log_file_for_archived_event_data(char *filename){
 		        }
 	        }
 
-	fclose(fp);
+	/* free memory and close the file */
+	free(input);
+	free(input2);
+	mmap_fclose(thefile);
 	
 	return;
         }
@@ -1386,6 +1428,10 @@ void convert_timeperiod_to_times(int type){
 		t2=current_time;
 		t1=current_time-(7*24*60*60);
 		break;
+	case TIMEPERIOD_LAST31DAYS:
+		t2=current_time;
+		t1=current_time-(31*24*60*60);
+		break;
 	default:
 		break;
 	        }
@@ -1455,7 +1501,7 @@ void add_archived_event(int event_type, time_t time_stamp, int entry_type, int s
 	archived_event *last_event=NULL;
 	archived_event *temp_event=NULL;
 	archived_event *new_event=NULL;
-	service *temp_service;
+	service *temp_service=NULL;
 	host *temp_host;
 
 
@@ -1482,7 +1528,7 @@ void add_archived_event(int event_type, time_t time_stamp, int entry_type, int s
 	        }
 		
 	/* find the host this entry is associated with */
-	temp_host=find_host(host_name,NULL);
+	temp_host=find_host(host_name);
 
 	/* check hostgroup match (valid filter for all reports) */
 	if(show_all_hostgroups==FALSE && is_host_member_of_hostgroup(target_hostgroup,temp_host)==FALSE)
@@ -1496,9 +1542,19 @@ void add_archived_event(int event_type, time_t time_stamp, int entry_type, int s
 			return;
 	        }
 
+	/* check servicegroup math (valid filter for all reports) */
+	if(event_type==AE_SERVICE_ALERT){
+		temp_service=find_service(host_name,svc_description);
+		if(show_all_servicegroups==FALSE && is_service_member_of_servicegroup(target_servicegroup,temp_service)==FALSE)
+			return;
+	         }
+	else{
+		if(show_all_servicegroups==FALSE && is_host_member_of_servicegroup(target_servicegroup,temp_host)==FALSE)
+			return;
+	         }
+
 	/* check authorization */
 	if(event_type==AE_SERVICE_ALERT){
-		temp_service=find_service(host_name,svc_description,NULL);
 		if(is_authorized_for_service(temp_service,&current_authdata)==FALSE)
 			return;
 	        }
@@ -1653,6 +1709,10 @@ void display_report(void){
 		display_host_alert_totals();
 		break;
 
+	case REPORT_SERVICEGROUP_ALERT_TOTALS:
+		display_servicegroup_alert_totals();
+		break;
+
 	case REPORT_SERVICE_ALERT_TOTALS:
 		display_service_alert_totals();
 		break;
@@ -1957,21 +2017,17 @@ void display_specific_hostgroup_alert_totals(hostgroup *grp){
 	if(grp==NULL)
 		return;
 
-	/* make sure the user is authorized to view at least one host in this hostgroup */
+	/* make sure the user is authorized to view this hostgroup */
 	if(is_authorized_for_hostgroup(grp,&current_authdata)==FALSE)
 		return;
 
 	/* process all events */
 	for(temp_event=event_list;temp_event!=NULL;temp_event=temp_event->next){
 
-		temp_host=find_host(temp_event->host_name,NULL);
+		temp_host=find_host(temp_event->host_name);
 		if(is_host_member_of_hostgroup(grp,temp_host)==FALSE)
 			continue;
 
-		/* make sure the user is authorized to view this host */
-		if(is_authorized_for_host(temp_host,&current_authdata)==FALSE)
-			continue;
-
 		/* host alerts */
 		if(temp_event->event_type==AE_HOST_ALERT){
 			if(temp_event->state_type==AE_SOFT_STATE){
@@ -2247,6 +2303,181 @@ void display_specific_host_alert_totals(host *hst){
         }
 
 
+/* displays servicegroup alert totals  */
+void display_servicegroup_alert_totals(void){
+	servicegroup *temp_servicegroup;
+
+	/**************************/
+	/**** SERVICEGROUP TOTALS ****/
+	/**************************/
+
+	printf("<BR>\n");
+
+	printf("<DIV ALIGN=CENTER>\n");
+	printf("<DIV ALIGN=CENTER CLASS='dataSubTitle'>Totals By Servicegroup</DIV>\n");
+
+	if(show_all_servicegroups==FALSE)
+		display_specific_servicegroup_alert_totals(target_servicegroup);
+	else{
+		for(temp_servicegroup=servicegroup_list;temp_servicegroup!=NULL;temp_servicegroup=temp_servicegroup->next)
+			display_specific_servicegroup_alert_totals(temp_servicegroup);
+	        }
+
+	printf("</DIV>\n");
+	
+	return;
+        }
+
+
+/* displays alert totals for a specific servicegroup */
+void display_specific_servicegroup_alert_totals(servicegroup *grp){
+	int hard_host_up_alerts=0;
+	int soft_host_up_alerts=0;
+	int hard_host_down_alerts=0;
+	int soft_host_down_alerts=0;
+	int hard_host_unreachable_alerts=0;
+	int soft_host_unreachable_alerts=0;
+	int hard_service_ok_alerts=0;
+	int soft_service_ok_alerts=0;
+	int hard_service_warning_alerts=0;
+	int soft_service_warning_alerts=0;
+	int hard_service_unknown_alerts=0;
+	int soft_service_unknown_alerts=0;
+	int hard_service_critical_alerts=0;
+	int soft_service_critical_alerts=0;
+	archived_event *temp_event;
+	host *temp_host;
+	service *temp_service;
+
+	if(grp==NULL)
+		return;
+
+	/* make sure the user is authorized to view this servicegroup */
+	if(is_authorized_for_servicegroup(grp,&current_authdata)==FALSE)
+		return;
+
+	/* process all events */
+	for(temp_event=event_list;temp_event!=NULL;temp_event=temp_event->next){
+
+		if(temp_event->event_type==AE_HOST_ALERT){
+
+			temp_host=find_host(temp_event->host_name);
+			if(is_host_member_of_servicegroup(grp,temp_host)==FALSE)
+				continue;
+		        }
+		else{
+
+			temp_service=find_service(temp_event->host_name,temp_event->service_description);
+			if(is_service_member_of_servicegroup(grp,temp_service)==FALSE)
+				continue;
+		        }
+
+		/* host alerts */
+		if(temp_event->event_type==AE_HOST_ALERT){
+			if(temp_event->state_type==AE_SOFT_STATE){
+				if(temp_event->entry_type==AE_HOST_UP)
+					soft_host_up_alerts++;
+				else if(temp_event->entry_type==AE_HOST_DOWN)
+					soft_host_down_alerts++;
+				else if(temp_event->entry_type==AE_HOST_UNREACHABLE)
+					soft_host_unreachable_alerts++;
+			        }
+			else{
+				if(temp_event->entry_type==AE_HOST_UP)
+					hard_host_up_alerts++;
+				else if(temp_event->entry_type==AE_HOST_DOWN)
+					hard_host_down_alerts++;
+				else if(temp_event->entry_type==AE_HOST_UNREACHABLE)
+					hard_host_unreachable_alerts++;
+			        }
+		        }
+
+		/* service alerts */
+		else{
+			if(temp_event->state_type==AE_SOFT_STATE){
+				if(temp_event->entry_type==AE_SERVICE_OK)
+					soft_service_ok_alerts++;
+				else if(temp_event->entry_type==AE_SERVICE_WARNING)
+					soft_service_warning_alerts++;
+				else if(temp_event->entry_type==AE_SERVICE_UNKNOWN)
+					soft_service_unknown_alerts++;
+				else if(temp_event->entry_type==AE_SERVICE_CRITICAL)
+					soft_service_critical_alerts++;
+			        }
+			else{
+				if(temp_event->entry_type==AE_SERVICE_OK)
+					hard_service_ok_alerts++;
+				else if(temp_event->entry_type==AE_SERVICE_WARNING)
+					hard_service_warning_alerts++;
+				else if(temp_event->entry_type==AE_SERVICE_UNKNOWN)
+					hard_service_unknown_alerts++;
+				else if(temp_event->entry_type==AE_SERVICE_CRITICAL)
+					hard_service_critical_alerts++;
+			        }
+		        }
+	        }
+
+
+	printf("<BR>\n");
+	printf("<TABLE BORDER=1 CELLSPACING=0 CELLPADDING=0 CLASS='reportDataEven'><TR><TD>\n");
+	printf("<TABLE BORDER=0>\n");
+
+	printf("<TR><TD COLSPAN=2 ALIGN=CENTER CLASS='dataSubTitle'>Servicegroup '%s' (%s)</TD></TR>\n",grp->group_name,grp->alias);
+
+	printf("<TR>\n");
+
+	if(alert_types & AE_HOST_ALERT){
+
+		printf("<TD ALIGN=CENTER VALIGN=TOP>\n");
+
+		printf("<DIV ALIGN=CENTER CLASS='dataSubTitle'>Host Alerts</DIV>\n");
+
+		printf("<DIV ALIGN=CENTER>\n");
+		printf("<TABLE BORDER=0 CLASS='data'>\n");
+		printf("<TR><TH CLASS='data'>State</TH><TH CLASS='data'>Soft Alerts</TH><TH CLASS='data'>Hard Alerts</TH><TH CLASS='data'>Total Alerts</TH></TR>\n");
+
+		printf("<TR CLASS='dataOdd'><TD CLASS='hostUP'>UP</TD><TD CLASS='dataOdd'>%d</TD><TD CLASS='dataOdd'>%d</TD><TD CLASS='dataOdd'>%d</TD></TR>\n",soft_host_up_alerts,hard_host_up_alerts,soft_host_up_alerts+hard_host_up_alerts);
+		printf("<TR CLASS='dataEven'><TD CLASS='hostDOWN'>DOWN</TD><TD CLASS='dataEven'>%d</TD><TD CLASS='dataEven'>%d</TD><TD CLASS='dataEven'>%d</TD></TR>\n",soft_host_down_alerts,hard_host_down_alerts,soft_host_down_alerts+hard_host_down_alerts);
+		printf("<TR CLASS='dataOdd'><TD CLASS='hostUNREACHABLE'>UNREACHABLE</TD><TD CLASS='dataOdd'>%d</TD><TD CLASS='dataOdd'>%d</TD><TD CLASS='dataOdd'>%d</TD></TR>\n",soft_host_unreachable_alerts,hard_host_unreachable_alerts,soft_host_unreachable_alerts+hard_host_unreachable_alerts);
+		printf("<TR CLASS='dataEven'><TD CLASS='dataEven'>All States</TD><TD CLASS='dataEven'>%d</TD><TD CLASS='dataEven'>%d</TD><TD CLASS='dataEven'><B>%d</B></TD></TR>\n",soft_host_up_alerts+soft_host_down_alerts+soft_host_unreachable_alerts,hard_host_up_alerts+hard_host_down_alerts+hard_host_unreachable_alerts,soft_host_up_alerts+hard_host_up_alerts+soft_host_down_alerts+hard_host_down_alerts+soft_host_unreachable_alerts+hard_host_unreachable_alerts);
+
+		printf("</TABLE>\n");
+		printf("</DIV>\n");
+
+		printf("</TD>\n");
+	        }
+
+	if(alert_types & AE_SERVICE_ALERT){
+
+		printf("<TD ALIGN=CENTER VALIGN=TOP>\n");
+
+		printf("<DIV ALIGN=CENTER CLASS='dataSubTitle'>Service Alerts</DIV>\n");
+
+		printf("<DIV ALIGN=CENTER>\n");
+		printf("<TABLE BORDER=0 CLASS='data'>\n");
+		printf("<TR><TH CLASS='data'>State</TH><TH CLASS='data'>Soft Alerts</TH><TH CLASS='data'>Hard Alerts</TH><TH CLASS='data'>Total Alerts</TH></TR>\n");
+
+		printf("<TR CLASS='dataOdd'><TD CLASS='serviceOK'>OK</TD><TD CLASS='dataOdd'>%d</TD><TD CLASS='dataOdd'>%d</TD><TD CLASS='dataOdd'>%d</TD></TR>\n",soft_service_ok_alerts,hard_service_ok_alerts,soft_service_ok_alerts+hard_service_ok_alerts);
+		printf("<TR CLASS='dataEven'><TD CLASS='serviceWARNING'>WARNING</TD><TD CLASS='dataEven'>%d</TD><TD CLASS='dataEven'>%d</TD><TD CLASS='dataEven'>%d</TD></TR>\n",soft_service_warning_alerts,hard_service_warning_alerts,soft_service_warning_alerts+hard_service_warning_alerts);
+		printf("<TR CLASS='dataOdd'><TD CLASS='serviceUNKNOWN'>UNKNOWN</TD><TD CLASS='dataOdd'>%d</TD><TD CLASS='dataOdd'>%d</TD><TD CLASS='dataOdd'>%d</TD></TR>\n",soft_service_unknown_alerts,hard_service_unknown_alerts,soft_service_unknown_alerts+hard_service_unknown_alerts);
+		printf("<TR CLASS='dataEven'><TD CLASS='serviceCRITICAL'>CRITICAL</TD><TD CLASS='dataEven'>%d</TD><TD CLASS='dataEven'>%d</TD><TD CLASS='dataEven'>%d</TD></TR>\n",soft_service_critical_alerts,hard_service_critical_alerts,soft_service_critical_alerts+hard_service_critical_alerts);
+		printf("<TR CLASS='dataOdd'><TD CLASS='dataOdd'>All States</TD><TD CLASS='dataOdd'>%d</TD><TD CLASS='dataOdd'>%d</TD><TD CLASS='dataOdd'><B>%d</B></TD></TR>\n",soft_service_ok_alerts+soft_service_warning_alerts+soft_service_unknown_alerts+soft_service_critical_alerts,hard_service_ok_alerts+hard_service_warning_alerts+hard_service_unknown_alerts+hard_service_critical_alerts,soft_service_ok_alerts+soft_service_warning_alerts+soft_service_unknown_alerts+soft_service_critical_alerts+hard_service_ok_alerts+hard_service_warning_alerts+hard_service_unknown_alerts+hard_service_critical_alerts);
+
+		printf("</TABLE>\n");
+		printf("</DIV>\n");
+
+		printf("</TD>\n");
+	        }
+
+	printf("</TR>\n");
+
+	printf("</TABLE>\n");
+	printf("</TD></TR></TABLE>\n");
+
+	return;
+        }
+
+
 
 /* displays service alert totals  */
 void display_service_alert_totals(void){
@@ -2291,7 +2522,7 @@ void display_specific_service_alert_totals(service *svc){
 		return;
 
 	if(show_all_hostgroups==FALSE && target_hostgroup!=NULL){
-		temp_host=find_host(svc->host_name,NULL);
+		temp_host=find_host(svc->host_name);
 		if(is_host_member_of_hostgroup(target_hostgroup,temp_host)==FALSE)
 			return;
 	        }
