@@ -3,16 +3,16 @@
  * TAC.C - Nagios Tactical Monitoring Overview CGI
  *
  * Copyright (c) 2001-2004 Ethan Galstad (nagios@nagios.org)
- * Last Modified: 01-05-2004
+ * Last Modified: 11-05-2004
  *
  * This CGI program will display the contents of the Nagios
  * log file.
  *
+ * License:
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU General Public License version 2 as
+ * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -24,16 +24,14 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  ***********************************************************************/
 
-#include "../common/config.h"
-#include "../common/locations.h"
-#include "../common/common.h"
-#include "../common/objects.h"
-#include "../common/statusdata.h"
+#include "../include/config.h"
+#include "../include/common.h"
+#include "../include/objects.h"
+#include "../include/statusdata.h"
 
-#include "getcgi.h"
-#include "cgiutils.h"
-#include "auth.h"
-#include "lifo.h"
+#include "../include/getcgi.h"
+#include "../include/cgiutils.h"
+#include "../include/cgiauth.h"
 
 
 #define HEALTH_WARNING_PERCENTAGE       90
@@ -62,11 +60,10 @@ extern char *host_down_sound;
 extern char *host_unreachable_sound;
 extern char *normal_sound;
 
+extern host *host_list;
 extern hostgroup *hostgroup_list;
 extern hoststatus *hoststatus_list;
 extern servicestatus *servicestatus_list;
-extern host *host_list;
-extern service *service_list;
 
 extern int enable_notifications;
 extern int execute_service_checks;
@@ -78,6 +75,7 @@ extern int nagios_process_state;
 
 
 
+
 void analyze_status_data(void);
 void display_tac_overview(void);
 
@@ -112,17 +110,27 @@ double percent_host_health=0.0;
 int total_hosts=0;
 int total_services=0;
 
-int total_active_checks=0;
-int total_passive_checks=0;
+int total_active_service_checks=0;
+int total_active_host_checks=0;
+int total_passive_service_checks=0;
+int total_passive_host_checks=0;
 
-int min_execution_time=-1;
-int max_execution_time=-1;
-unsigned long total_execution_time=0L;
-double average_execution_time=-1.0;
-int min_latency=-1;
-int max_latency=-1;
-unsigned long total_latency=0L;
-double average_latency=-1.0;
+double min_service_execution_time=-1.0;
+double max_service_execution_time=-1.0;
+double total_service_execution_time=0.0;
+double average_service_execution_time=-1.0;
+double min_host_execution_time=-1.0;
+double max_host_execution_time=-1.0;
+double total_host_execution_time=0.0;
+double average_host_execution_time=-1.0;
+double min_service_latency=-1.0;
+double max_service_latency=-1.0;
+double total_service_latency=0.0;
+double average_service_latency=-1.0;
+double min_host_latency=-1.0;
+double max_host_latency=-1.0;
+double total_host_latency=0.0;
+double average_host_latency=-1.0;
 
 int flapping_services=0;
 int flapping_hosts=0;
@@ -135,6 +143,7 @@ int event_handler_disabled_hosts=0;
 int active_checks_disabled_services=0;
 int active_checks_disabled_hosts=0;
 int passive_checks_disabled_services=0;
+int passive_checks_disabled_hosts=0;
 
 int hosts_pending=0;
 int hosts_pending_disabled=0;
@@ -190,10 +199,10 @@ int main(void){
 	reset_cgi_vars();
 
 	/* read the CGI configuration file */
-	result=read_cgi_config_file(DEFAULT_CGI_CONFIG_FILE);
+	result=read_cgi_config_file(get_cgi_config_location());
 	if(result==ERROR){
 		document_header(FALSE);
-		cgi_config_file_error(DEFAULT_CGI_CONFIG_FILE);
+		cgi_config_file_error(get_cgi_config_location());
 		document_footer();
 		return ERROR;
 	        }
@@ -217,7 +226,7 @@ int main(void){
                 }
 
 	/* read all status data */
-	result=read_all_status_data(DEFAULT_CGI_CONFIG_FILE,READ_ALL_STATUS_DATA);
+	result=read_all_status_data(get_cgi_config_location(),READ_ALL_STATUS_DATA);
 	if(result==ERROR){
 		document_header(FALSE);
 		status_data_error();
@@ -325,8 +334,10 @@ void document_header(int use_stylesheet){
 	printf("Nagios Tactical Monitoring Overview\n");
 	printf("</TITLE>\n");
 
-	if(use_stylesheet==TRUE)
+	if(use_stylesheet==TRUE){
+		printf("<LINK REL='stylesheet' TYPE='text/css' HREF='%s%s'>\n",url_stylesheets_path,COMMON_CSS);
 		printf("<LINK REL='stylesheet' TYPE='text/css' HREF='%s%s'>\n",url_stylesheets_path,TAC_CSS);
+	        }
 
 	printf("</HEAD>\n");
 	printf("<BODY CLASS='tac' marginwidth=2 marginheight=2 topmargin=0 leftmargin=0 rightmargin=0>\n");
@@ -375,7 +386,7 @@ int process_cgivars(void){
 		else if(!strcmp(variables[x],"noheader"))
 			display_header=FALSE;
 
-		/* we recieved an invalid argument */
+		/* we received an invalid argument */
 		else
 			error=TRUE;
 	
@@ -401,7 +412,7 @@ void analyze_status_data(void){
 	for(temp_servicestatus=servicestatus_list;temp_servicestatus!=NULL;temp_servicestatus=temp_servicestatus->next){
 
 		/* see if user is authorized to view this service */
-		temp_service=find_service(temp_servicestatus->host_name,temp_servicestatus->description,NULL);
+		temp_service=find_service(temp_servicestatus->host_name,temp_servicestatus->description);
 		if(is_authorized_for_service(temp_service,&current_authdata)==FALSE)
 			continue;
 
@@ -434,7 +445,7 @@ void analyze_status_data(void){
 
 		problem=TRUE;
 
-		if(temp_servicestatus->status==SERVICE_OK || temp_servicestatus->status==SERVICE_RECOVERY){
+		if(temp_servicestatus->status==SERVICE_OK){
 			if(temp_servicestatus->checks_enabled==FALSE)
 				services_ok_disabled++;
 			else
@@ -532,23 +543,23 @@ void analyze_status_data(void){
 		/* calculate execution time and latency stats */
 		if(temp_servicestatus->check_type==SERVICE_CHECK_ACTIVE){
 
-			total_active_checks++;
+			total_active_service_checks++;
 
-			if(min_latency==-1 || temp_servicestatus->latency<min_latency)
-				min_latency=temp_servicestatus->latency;
-			if(max_latency==-1 || temp_servicestatus->latency>max_latency)
-				max_latency=temp_servicestatus->latency;
+			if(min_service_latency==-1.0 || temp_servicestatus->latency<min_service_latency)
+				min_service_latency=temp_servicestatus->latency;
+			if(max_service_latency==-1.0 || temp_servicestatus->latency>max_service_latency)
+				max_service_latency=temp_servicestatus->latency;
 
-			if(min_execution_time==-1 || temp_servicestatus->execution_time<min_execution_time)
-				min_execution_time=temp_servicestatus->execution_time;
-			if(max_execution_time==-1 || temp_servicestatus->execution_time>max_execution_time)
-				max_execution_time=temp_servicestatus->execution_time;
+			if(min_service_execution_time==-1.0 || temp_servicestatus->execution_time<min_service_execution_time)
+				min_service_execution_time=temp_servicestatus->execution_time;
+			if(max_service_execution_time==-1.0 || temp_servicestatus->execution_time>max_service_execution_time)
+				max_service_execution_time=temp_servicestatus->execution_time;
 
-			total_latency+=temp_servicestatus->latency;
-			total_execution_time+=temp_servicestatus->execution_time;
+			total_service_latency+=temp_servicestatus->latency;
+			total_service_execution_time+=temp_servicestatus->execution_time;
 		        }
 		else
-			total_passive_checks++;
+			total_passive_service_checks++;
 
 
 		total_services++;
@@ -560,7 +571,7 @@ void analyze_status_data(void){
 	for(temp_hoststatus=hoststatus_list;temp_hoststatus!=NULL;temp_hoststatus=temp_hoststatus->next){
 
 		/* see if user is authorized to view this host */
-		temp_host=find_host(temp_hoststatus->host_name,NULL);
+		temp_host=find_host(temp_hoststatus->host_name);
 		if(is_authorized_for_host(temp_host,&current_authdata)==FALSE)
 			continue;
 
@@ -584,6 +595,10 @@ void analyze_status_data(void){
 		if(temp_hoststatus->checks_enabled==FALSE)
 			active_checks_disabled_hosts++;
 
+		/* passive check acceptance */
+		if(temp_hoststatus->accept_passive_host_checks==FALSE)
+			passive_checks_disabled_hosts++;
+
 
 		/********* CHECK STATUS ********/
 
@@ -646,6 +661,27 @@ void analyze_status_data(void){
 		if(temp_hoststatus->status!=HOST_PENDING)
 			potential_host_health++;
 
+		/* check type stats */
+		if(temp_hoststatus->check_type==HOST_CHECK_ACTIVE){
+
+			total_active_host_checks++;
+
+			if(min_host_latency==-1.0 || temp_hoststatus->latency<min_host_latency)
+				min_host_latency=temp_hoststatus->latency;
+			if(max_host_latency==-1.0 || temp_hoststatus->latency>max_host_latency)
+				max_host_latency=temp_hoststatus->latency;
+
+			if(min_host_execution_time==-1.0 || temp_hoststatus->execution_time<min_host_execution_time)
+				min_host_execution_time=temp_hoststatus->execution_time;
+			if(max_host_execution_time==-1.0 || temp_hoststatus->execution_time>max_host_execution_time)
+				max_host_execution_time=temp_hoststatus->execution_time;
+
+			total_host_latency+=temp_hoststatus->latency;
+			total_host_execution_time+=temp_hoststatus->execution_time;
+		        }
+		else
+			total_passive_host_checks++;
+
 		total_hosts++;
 	        }
 
@@ -662,17 +698,29 @@ void analyze_status_data(void){
 	else
 		percent_host_health=((double)total_host_health/(double)potential_host_health)*100.0;
 
-	/* calculate latency */
-	if(total_latency==0L)
-		average_latency=0.0;
+	/* calculate service latency */
+	if(total_service_latency==0L)
+		average_service_latency=0.0;
 	else
-		average_latency=((double)total_latency/(double)total_active_checks);
+		average_service_latency=((double)total_service_latency/(double)total_active_service_checks);
 
-	/* calculate execution time */
-	if(total_execution_time==0L)
-		average_execution_time=0.0;
+	/* calculate host latency */
+	if(total_host_latency==0L)
+		average_host_latency=0.0;
 	else
-		average_execution_time=((double)total_execution_time/(double)total_active_checks);
+		average_host_latency=((double)total_host_latency/(double)total_active_host_checks);
+
+	/* calculate service execution time */
+	if(total_service_execution_time==0.0)
+		average_service_execution_time=0.0;
+	else
+		average_service_execution_time=((double)total_service_execution_time/(double)total_active_service_checks);
+
+	/* calculate host execution time */
+	if(total_host_execution_time==0.0)
+		average_host_execution_time=0.0;
+	else
+		average_host_execution_time=((double)total_host_execution_time/(double)total_active_host_checks);
 
 	return;
         }
@@ -697,7 +745,7 @@ void find_hosts_causing_outages(void){
 		if(temp_hoststatus->status!=HOST_UP && temp_hoststatus->status!=HOST_PENDING){
 
 			/* find the host entry */
-			temp_host=find_host(temp_hoststatus->host_name,NULL);
+			temp_host=find_host(temp_hoststatus->host_name);
 
 			if(temp_host==NULL)
 				continue;
@@ -872,20 +920,28 @@ void display_tac_overview(void){
 	printf("<td class='perfBox'>\n");
 	printf("<table border=0 cellspacing=4 cellspadding=0>\n");
 	printf("<tr>\n");
-	printf("<td align=left valign=center class='perfItem'><a href='%s?type=%d' class='perfItem'>Check Execution Time:</a></td>",EXTINFO_CGI,DISPLAY_PERFORMANCE);
-	printf("<td valign=top class='perfValue' nowrap><a href='%s?type=%d' class='perfValue'>%2d / %d / %2.3f sec</a></td>\n",EXTINFO_CGI,DISPLAY_PERFORMANCE,min_execution_time,max_execution_time,average_execution_time);
+	printf("<td align=left valign=center class='perfItem'><a href='%s?type=%d' class='perfItem'>Service Check Execution Time:</a></td>",EXTINFO_CGI,DISPLAY_PERFORMANCE);
+	printf("<td valign=top class='perfValue' nowrap><a href='%s?type=%d' class='perfValue'>%.2f / %.2f / %.3f sec</a></td>\n",EXTINFO_CGI,DISPLAY_PERFORMANCE,min_service_execution_time,max_service_execution_time,average_service_execution_time);
 	printf("</tr>\n");
 	printf("<tr>\n");
-	printf("<td align=left valign=center class='perfItem'><a href='%s?type=%d' class='perfItem'>Check Latency:</a></td>",EXTINFO_CGI,DISPLAY_PERFORMANCE);
-	printf("<td valign=top class='perfValue' nowrap><a href='%s?type=%d' class='perfValue'>%d / %d / %2.3f sec</a></td>\n",EXTINFO_CGI,DISPLAY_PERFORMANCE,min_latency,max_latency,average_latency);
+	printf("<td align=left valign=center class='perfItem'><a href='%s?type=%d' class='perfItem'>Service Check Latency:</a></td>",EXTINFO_CGI,DISPLAY_PERFORMANCE);
+	printf("<td valign=top class='perfValue' nowrap><a href='%s?type=%d' class='perfValue'>%.2f / %.2f / %.3f sec</a></td>\n",EXTINFO_CGI,DISPLAY_PERFORMANCE,min_service_latency,max_service_latency,average_service_latency);
 	printf("</tr>\n");
 	printf("<tr>\n");
-	printf("<td align=left valign=center class='perfItem'><a href='%s?host=all&serviceprops=%d' class='perfItem'># Active Checks:</a></td>",STATUS_CGI,SERVICE_ACTIVE_CHECK);
-	printf("<td valign=top class='perfValue' nowrap><a href='%s?host=all&serviceprops=%d' class='perfValue'>%d</a></td>\n",STATUS_CGI,SERVICE_ACTIVE_CHECK,total_active_checks);
+	printf("<td align=left valign=center class='perfItem'><a href='%s?type=%d' class='perfItem'>Host Check Execution Time:</a></td>",EXTINFO_CGI,DISPLAY_PERFORMANCE);
+	printf("<td valign=top class='perfValue' nowrap><a href='%s?type=%d' class='perfValue'>%.2f / %.2f / %.3f sec</a></td>\n",EXTINFO_CGI,DISPLAY_PERFORMANCE,min_host_execution_time,max_host_execution_time,average_host_execution_time);
 	printf("</tr>\n");
 	printf("<tr>\n");
-	printf("<td align=left valign=center class='perfItem'><a href='%s?host=all&serviceprops=%d' class='perfItem'># Passive Checks:</a></td>",STATUS_CGI,SERVICE_PASSIVE_CHECK);
-	printf("<td valign=top class='perfValue' nowrap><a href='%s?host=all&serviceprops=%d' class='perfValue'>%d</a></td>\n",STATUS_CGI,SERVICE_PASSIVE_CHECK,total_passive_checks);
+	printf("<td align=left valign=center class='perfItem'><a href='%s?type=%d' class='perfItem'>Host Check Latency:</a></td>",EXTINFO_CGI,DISPLAY_PERFORMANCE);
+	printf("<td valign=top class='perfValue' nowrap><a href='%s?type=%d' class='perfValue'>%.2f / %.2f / %2.3f sec</a></td>\n",EXTINFO_CGI,DISPLAY_PERFORMANCE,min_host_latency,max_host_latency,average_host_latency);
+	printf("</tr>\n");
+	printf("<tr>\n");
+	printf("<td align=left valign=center class='perfItem'><a href='%s?host=all&serviceprops=%d' class='perfItem'># Active Host / Service Checks:</a></td>",STATUS_CGI,SERVICE_ACTIVE_CHECK);
+	printf("<td valign=top class='perfValue' nowrap><a href='%s?hostgroup=all&hostprops=%d&style=hostdetail' class='perfValue'>%d</a> / <a href='%s?host=all&serviceprops=%d' class='perfValue'>%d</a></td>\n",STATUS_CGI,HOST_ACTIVE_CHECK,total_active_host_checks,STATUS_CGI,SERVICE_ACTIVE_CHECK,total_active_service_checks);
+	printf("</tr>\n");
+	printf("<tr>\n");
+	printf("<td align=left valign=center class='perfItem'><a href='%s?host=all&serviceprops=%d' class='perfItem'># Passive Host / Service Checks:</a></td>",STATUS_CGI,SERVICE_PASSIVE_CHECK);
+	printf("<td valign=top class='perfValue' nowrap><a href='%s?hostgroup=all&hostprops=%d&style=hostdetail' class='perfValue'>%d</a> / <a href='%s?host=all&serviceprops=%d' class='perfValue'>%d</a></td>\n",STATUS_CGI,HOST_PASSIVE_CHECK,total_passive_host_checks,STATUS_CGI,SERVICE_PASSIVE_CHECK,total_passive_service_checks);
 	printf("</tr>\n");
 	printf("</table>\n");
 	printf("</td>\n");
@@ -1006,11 +1062,11 @@ void display_tac_overview(void){
 	printf("<table border=0 cellspacing=4 cellspadding=0>\n");
 	printf("<tr>\n");
 	printf("<td align=left valign=center class='healthItem'>Host Health:</td>");
-	printf("<td valign=top width=100 class='healthBar'><img src='%s%s' border=0 width=%d height=20 alt='%2.1f%% Health'></td>\n",url_images_path,host_health_image,(percent_host_health<5.0)?5:(int)percent_host_health,percent_host_health);
+	printf("<td valign=top width=100 class='healthBar'><img src='%s%s' border=0 width=%d height=20 alt='%2.1f%% Health' title='%2.1f%% Health'></td>\n",url_images_path,host_health_image,(percent_host_health<5.0)?5:(int)percent_host_health,percent_host_health,percent_host_health);
 	printf("</tr>\n");
 	printf("<tr>\n");
 	printf("<td align=left valign=center class='healthItem'>Service Health:</td>");
-	printf("<td valign=top width=100 class='healthBar'><img src='%s%s' border=0 width=%d height=20 alt='%2.1f%% Health'></td>\n",url_images_path,service_health_image,(percent_service_health<5.0)?5:(int)percent_service_health,percent_service_health);
+	printf("<td valign=top width=100 class='healthBar'><img src='%s%s' border=0 width=%d height=20 alt='%2.1f%% Health' title='%2.1f%% Health'></td>\n",url_images_path,service_health_image,(percent_service_health<5.0)?5:(int)percent_service_health,percent_service_health,percent_service_health);
 	printf("</tr>\n");
 	printf("</table>\n");
 	printf("</td>\n");
@@ -1093,7 +1149,7 @@ void display_tac_overview(void){
 	printf("<table border=0 width=100%%>\n");
 
 	if(hosts_unreachable_unacknowledged>0)
-		printf("<tr><td width=100%% class='hostImportantProblem'><a href='%s?hostgroup=all&style=hostdetail&hoststatustypes=%d&hostprops=%d'>%d Unhandled Problems</a></td></tr>\n",STATUS_CGI,HOST_UNREACHABLE,HOST_NO_SCHEDULED_DOWNTIME|HOST_STATE_UNACKNOWLEDGED|HOST_CHECKS_ENABLED,hosts_unreachable_unacknowledged);
+		printf("<tr><td width=100%% class='hostImportantProblem'><a href='%s?host=all&style=hostdetail&hoststatustypes=%d&hostprops=%d'>%d Unhandled Problems</a></td></tr>\n",STATUS_CGI,HOST_UNREACHABLE,HOST_NO_SCHEDULED_DOWNTIME|HOST_STATE_UNACKNOWLEDGED|HOST_CHECKS_ENABLED,hosts_unreachable_unacknowledged);
 
 	if(hosts_unreachable_scheduled>0)
 		printf("<tr><td width=100%% class='hostUnimportantProblem'><a href='%s?hostgroup=all&style=hostdetail&hoststatustypes=%d&hostprops=%d'>%d Scheduled</a></td></tr>\n",STATUS_CGI,HOST_UNREACHABLE,HOST_SCHEDULED_DOWNTIME,hosts_unreachable_scheduled);
@@ -1161,8 +1217,10 @@ void display_tac_overview(void){
 	printf("</tr>\n");
 	printf("</table>\n");
 
+	/*
 	printf("</tr>\n");
 	printf("</table>\n");
+	*/
 
 	printf("</p>\n");
 
@@ -1180,7 +1238,7 @@ void display_tac_overview(void){
 
 	printf("<table class='tac' width=641 cellspacing=4 cellpadding=0 border=0>\n");
 
-	printf("<tr><td colspan=5 height=20 class='hostTitle'>&nbsp;Services</td></tr>\n");
+	printf("<tr><td colspan=5 height=20 class='serviceTitle'>&nbsp;Services</td></tr>\n");
 
 	printf("<tr>\n");
 	printf("<td class='serviceHeader' width=125><a href='%s?host=all&stye=detail&servicestatustypes=%d' class='serviceHeader'>%d Critical</a></td>\n",STATUS_CGI,SERVICE_CRITICAL,services_critical);
@@ -1222,7 +1280,6 @@ void display_tac_overview(void){
 
 	printf("</tr>\n");
 	printf("</table>\n");
-
 	printf("</td>\n");
 
 
@@ -1308,7 +1365,7 @@ void display_tac_overview(void){
 	printf("<table border=0 width=100%%>\n");
 
 	if(services_ok_disabled>0)
-		printf("<tr><td width=100%% class='serviceUnimportantProblem'><a href='%s?host=all&type=detail&servicestatustypes=%d&serviceprops=%d'>%d Disabled</a></td></tr>\n",STATUS_CGI,SERVICE_OK|SERVICE_RECOVERY,SERVICE_CHECKS_DISABLED,services_ok_disabled);
+		printf("<tr><td width=100%% class='serviceUnimportantProblem'><a href='%s?host=all&type=detail&servicestatustypes=%d&serviceprops=%d'>%d Disabled</a></td></tr>\n",STATUS_CGI,SERVICE_OK,SERVICE_CHECKS_DISABLED,services_ok_disabled);
 
 	printf("</table>\n");
 	printf("</td>\n");
@@ -1363,19 +1420,19 @@ void display_tac_overview(void){
 	printf("<tr><td colspan=5 height=20 class='featureTitle'>&nbsp;Monitoring Features</td></tr>\n");
 
 	printf("<tr>\n");
-	printf("<td class='featureHeader' width=125>Flap Detection</td>\n");
-	printf("<td class='featureHeader' width=125>Notifications</td>\n");
-	printf("<td class='featureHeader' width=125>Event Handlers</td>\n");
-	printf("<td class='featureHeader' width=125>Active Checks</td>\n");
-	printf("<td class='featureHeader' width=125>Passive Checks</td>\n");
+	printf("<td class='featureHeader' width=135>Flap Detection</td>\n");
+	printf("<td class='featureHeader' width=135>Notifications</td>\n");
+	printf("<td class='featureHeader' width=135>Event Handlers</td>\n");
+	printf("<td class='featureHeader' width=135>Active Checks</td>\n");
+	printf("<td class='featureHeader' width=135>Passive Checks</td>\n");
 	printf("</tr>\n");
 
 	printf("<tr>\n");
 
 	printf("<td valign=top>\n");
-	printf("<table border=0 width=125 cellspacing=0 cellpadding=0>\n");
+	printf("<table border=0 width=135 cellspacing=0 cellpadding=0>\n");
 	printf("<tr>\n");
-	printf("<td valign=top><a href='%s?cmd_typ=%d'><img src='%s%s' border=0 alt='Flap Detection %s'></a></td>\n",COMMAND_CGI,(enable_flap_detection==TRUE)?CMD_DISABLE_FLAP_DETECTION:CMD_ENABLE_FLAP_DETECTION,url_images_path,(enable_flap_detection==TRUE)?TAC_ENABLED_ICON:TAC_DISABLED_ICON,(enable_flap_detection==TRUE)?"Enabled":"Disabled");
+	printf("<td valign=top><a href='%s?cmd_typ=%d'><img src='%s%s' border=0 alt='Flap Detection %s' title='Flap Detection %s'></a></td>\n",COMMAND_CGI,(enable_flap_detection==TRUE)?CMD_DISABLE_FLAP_DETECTION:CMD_ENABLE_FLAP_DETECTION,url_images_path,(enable_flap_detection==TRUE)?TAC_ENABLED_ICON:TAC_DISABLED_ICON,(enable_flap_detection==TRUE)?"Enabled":"Disabled",(enable_flap_detection==TRUE)?"Enabled":"Disabled");
 	printf("<Td width=10>&nbsp;</td>\n");
 	if(enable_flap_detection==TRUE){
 		printf("<Td valign=top width=100%% class='featureEnabledFlapDetection'>\n");
@@ -1414,9 +1471,9 @@ void display_tac_overview(void){
 
 
 	printf("<td valign=top>\n");
-	printf("<table border=0 width=125 cellspacing=0 cellpadding=0>\n");
+	printf("<table border=0 width=135 cellspacing=0 cellpadding=0>\n");
 	printf("<tr>\n");
-	printf("<td valign=top><a href='%s?cmd_typ=%d'><img src='%s%s' border=0 alt='Notifications %s'></a></td>\n",COMMAND_CGI,(enable_notifications==TRUE)?CMD_DISABLE_NOTIFICATIONS:CMD_ENABLE_NOTIFICATIONS,url_images_path,(enable_notifications==TRUE)?TAC_ENABLED_ICON:TAC_DISABLED_ICON,(enable_notifications==TRUE)?"Enabled":"Disabled");
+	printf("<td valign=top><a href='%s?cmd_typ=%d'><img src='%s%s' border=0 alt='Notifications %s' title='Notifications %s'></a></td>\n",COMMAND_CGI,(enable_notifications==TRUE)?CMD_DISABLE_NOTIFICATIONS:CMD_ENABLE_NOTIFICATIONS,url_images_path,(enable_notifications==TRUE)?TAC_ENABLED_ICON:TAC_DISABLED_ICON,(enable_notifications==TRUE)?"Enabled":"Disabled",(enable_notifications==TRUE)?"Enabled":"Disabled");
 	printf("<Td width=10>&nbsp;</td>\n");
 	if(enable_notifications==TRUE){
 		printf("<Td valign=top width=100%% class='featureEnabledNotifications'>\n");
@@ -1446,9 +1503,9 @@ void display_tac_overview(void){
 
 
 	printf("<td valign=top>\n");
-	printf("<table border=0 width=125 cellspacing=0 cellpadding=0>\n");
+	printf("<table border=0 width=135 cellspacing=0 cellpadding=0>\n");
 	printf("<tr>\n");
-	printf("<td valign=top><a href='%s?cmd_typ=%d'><img src='%s%s' border=0 alt='Event Handlers %s'></a></td>\n",COMMAND_CGI,(enable_event_handlers==TRUE)?CMD_DISABLE_EVENT_HANDLERS:CMD_ENABLE_EVENT_HANDLERS,url_images_path,(enable_event_handlers==TRUE)?TAC_ENABLED_ICON:TAC_DISABLED_ICON,(enable_event_handlers==TRUE)?"Enabled":"Disabled");
+	printf("<td valign=top><a href='%s?cmd_typ=%d'><img src='%s%s' border=0 alt='Event Handlers %s' title='Event Handlers %s'></a></td>\n",COMMAND_CGI,(enable_event_handlers==TRUE)?CMD_DISABLE_EVENT_HANDLERS:CMD_ENABLE_EVENT_HANDLERS,url_images_path,(enable_event_handlers==TRUE)?TAC_ENABLED_ICON:TAC_DISABLED_ICON,(enable_event_handlers==TRUE)?"Enabled":"Disabled",(enable_event_handlers==TRUE)?"Enabled":"Disabled");
 	printf("<Td width=10>&nbsp;</td>\n");
 	if(enable_event_handlers==TRUE){
 		printf("<Td valign=top width=100%% class='featureEnabledHandlers'>\n");
@@ -1478,9 +1535,9 @@ void display_tac_overview(void){
 
 
 	printf("<td valign=top>\n");
-	printf("<table border=0 width=125 cellspacing=0 cellpadding=0>\n");
+	printf("<table border=0 width=135 cellspacing=0 cellpadding=0>\n");
 	printf("<tr>\n");
-	printf("<td valign=top><a href='%s?cmd_typ=%d'><img src='%s%s' border='0' alt='Active Checks %s'></a></td>\n",COMMAND_CGI,(execute_service_checks==TRUE)?CMD_STOP_EXECUTING_SVC_CHECKS:CMD_START_EXECUTING_SVC_CHECKS,url_images_path,(execute_service_checks==TRUE)?TAC_ENABLED_ICON:TAC_DISABLED_ICON,(execute_service_checks==TRUE)?"Enabled":"Disabled");
+	printf("<td valign=top><a href='%s?type=%d'><img src='%s%s' border='0' alt='Active Checks %s' title='Active Checks %s'></a></td>\n",EXTINFO_CGI,DISPLAY_PROCESS_INFO,url_images_path,(execute_service_checks==TRUE)?TAC_ENABLED_ICON:TAC_DISABLED_ICON,(execute_service_checks==TRUE)?"Enabled":"Disabled",(execute_service_checks==TRUE)?"Enabled":"Disabled");
 	printf("<Td width=10>&nbsp;</td>\n");
 	if(execute_service_checks==TRUE){
 		printf("<Td valign=top width=100%% class='featureEnabledActiveChecks'>\n");
@@ -1510,20 +1567,25 @@ void display_tac_overview(void){
 
 
 	printf("<td valign=top>\n");
-	printf("<table border=0 width=125 cellspacing=0 cellpadding=0>\n");
+	printf("<table border=0 width=135 cellspacing=0 cellpadding=0>\n");
 	printf("<tr>\n");
-	printf("<td valign=top><a href='%s?cmd_typ=%d'><img src='%s%s' border='0' alt='Passive Checks %s'></a></td>\n",COMMAND_CGI,(accept_passive_service_checks==TRUE)?CMD_STOP_ACCEPTING_PASSIVE_SVC_CHECKS:CMD_START_ACCEPTING_PASSIVE_SVC_CHECKS,url_images_path,(accept_passive_service_checks==TRUE)?TAC_ENABLED_ICON:TAC_DISABLED_ICON,(accept_passive_service_checks==TRUE)?"Enabled":"Disabled");
+	printf("<td valign=top><a href='%s?type=%d'><img src='%s%s' border='0' alt='Passive Checks %s' title='Passive Checks %s'></a></td>\n",EXTINFO_CGI,DISPLAY_PROCESS_INFO,url_images_path,(accept_passive_service_checks==TRUE)?TAC_ENABLED_ICON:TAC_DISABLED_ICON,(accept_passive_service_checks==TRUE)?"Enabled":"Disabled",(accept_passive_service_checks==TRUE)?"Enabled":"Disabled");
 	printf("<Td width=10>&nbsp;</td>\n");
 	if(accept_passive_service_checks==TRUE){
 
 		printf("<Td valign=top width=100%% class='featureEnabledPassiveChecks'>\n");
-		printf("<table border=0 width=100%% cellspacing=0 cellpadding=0>\n");
+		printf("<table border=0 width=100%%>\n");
 
 		if(passive_checks_disabled_services>0)
 			printf("<tr><td width=100%% class='featureItemDisabledPassiveServiceChecks'><a href='%s?host=all&type=detail&serviceprops=%d'>%d Service%s Disabled</a></td></tr>\n",STATUS_CGI,SERVICE_PASSIVE_CHECKS_DISABLED,passive_checks_disabled_services,(passive_checks_disabled_services==1)?"":"s");
 		else
 			printf("<tr><td width=100%% class='featureItemEnabledPassiveServiceChecks'>All Services Enabled</td></tr>\n");
 
+		if(passive_checks_disabled_hosts>0)
+			printf("<tr><td width=100%% class='featureItemDisabledPassiveHostChecks'><a href='%s?host=all&type=detail&hostprops=%d'>%d Host%s Disabled</a></td></tr>\n",STATUS_CGI,HOST_PASSIVE_CHECKS_DISABLED,passive_checks_disabled_hosts,(passive_checks_disabled_hosts==1)?"":"s");
+		else
+			printf("<tr><td width=100%% class='featureItemEnabledPassiveHostChecks'>All Hosts Enabled</td></tr>\n");
+
 		printf("</table>\n");
 		printf("</td>\n");
 	        }
